///////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2008 Cinterion Wireless Modules GmbH
//
// This software is protected by international intellectual property laws and
// treaties.
// Cinterion Wireless Modules GmbH ("Cinterion") grants to customer a limited,
// non-exclusive, non-transferable, royalty-free license to use the source code
// as a sample implementation and for the purpose of evaluation, analysis and
// to provide feedback thereon to Cinterion. Any right, title and interest in
// and to the source code, other than those expressly granted to the customer
// under this agreement, shall remain vested with Cinterion. This license may
// be terminated by Cinterion at any time in writing. Customer undertakes not
// to provide third parties with the source code. In particular, Customer is
// not allowed to sell the source code.
// The information contained in this document is considered the CONFIDENTIAL
// and PROPRIETARY information of Cinterion Wireless Modules GmbH and may not
// be disclosed or discussed with anyone who is not employed by Cinterion
// Wireless Modules GmbH, unless the individual/ company:
//
// i)  has an express need to know such information, and
//
// ii) disclosure of information is subject to the terms of a duly executed
//     Confidentiality and Non-Disclosure Agreement between Cinterion Wireless
//     Modules GmbH and the individual / company.
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
// EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
// CINTERION, ITS LEGAL REPRESENTATIVES AND VICARIOUS AGENTS SHALL 
// IRRESPECTIVE OF THE LEGAL GROUND  ONLY BE LIABLE FOR DAMAGES IF THE DAMAGE
// WAS CAUSED THROUGH CULPABLE BREACH OF A MAJOR CONTRACTUAL OBLIGATION
// (CARDINAL DUTY), I.E. A DUTY THE FULFILMENT OF WHICH ALLOWS THE PROPER
// EXECUTION OF THE RESPECTIVE AGREEMENT IN THE FIRST PLACE OR THE BREACH OF
// WHICH PUTS THE ACHIEVEMENT OF THE PURPOSE OF THE AGREEMENT AT STAKE,
// RESPECTIVELY, AND ON THE FULFILMENT OF WHICH THE RECIPIENT THEREFORE MAY
// RELY ON OR WAS CAUSED BY GROSS NEGLIGENCE OR INTENTIONALLY. ANY FURTHER
// LIABILITY FOR DAMAGES SHALL  IRRESPECTIVE OF THE LEGAL GROUND  BE
// EXCLUDED. IN THE EVENT THAT CINTERION IS LIABLE FOR THE VIOLATION OF A MAJOR
// CONTRACTUAL OBLIGATION IN THE ABSENCE OF GROSS NEGLIGENCE OR WILFUL CONDUCT,
// SUCH LIABILITY FOR DAMAGE SHALL BE LIMITED TO AN EXTENT WHICH, AT THE TIME
// WHEN THE RESPECTIVE AGREEMENT IS CONCLUDED, CINTERION SHOULD NORMALLY EXPECT
// TO ARISE DUE TO CIRCUMSTANCES THAT THE PARTIES HAD KNOWLEDGE OF AT SUCH
// POINT IN TIME. CINTERION SHALL IN NO EVENT BE LIABLE FOR INDIRECT AND
// CONSEQUENTIAL DAMAGES OR LOSS OF PROFIT. CINTERION SHALL IN NO EVENT BE
// LIABLE FOR AN AMOUNT EXCEEDING  20,000.00 PER EVENT OF DAMAGE. WITHIN THE
// BUSINESS RELATIONSHIP THE OVERALL LIABILITY SHALL BE LIMITED TO A TOTAL OF
//  100,000.00. CLAIMS FOR DAMAGES SHALL BECOME TIME-BARRED AFTER ONE YEAR AS
// OF THE BEGINNING OF THE STATUTORY LIMITATION PERIOD. IRRESPECTIVE OF THE
// LICENSEES KNOWLEDGE OR GROSS NEGLIGENT LACK OF KNOWLEDGE OF THE
// CIRCUMSTANCES GIVING RISE FOR A LIABILITY ANY CLAIMS SHALL BECOME
// TIME-BARRED AFTER FIVE YEARS AS OF THE LIABILITY AROSE. THE AFOREMENTIONED
// LIMITATION OR EXCLUSION OF LIABILITY SHALL NOT APPLY IN THE CASE OF CULPABLE
// INJURY TO LIFE, BODY OR HEALTH, IN CASE OF INTENTIONAL ACTS, UNDER THE
// LIABILITY PROVISIONS OF THE GERMAN PRODUCT LIABILITY ACT
// (PRODUKTHAFTUNGSGESETZ) OR IN CASE OF A CONTRACTUALLY AGREED OBLIGATION TO
// ASSUME LIABILITY IRRESPECTIVE OF ANY FAULT (GUARANTEE).
//
// IN THE EVENT OF A CONFLICT BETWEEN THE PROVISIONS OF THIS AGREEMENT AND
// ANOTHER AGREEMENT REGARDING THE SOURCE CODE (EXCEPT THE GENERAL TERMS AND
// CONDITIONS OF CINTERION) THE OTHER AGREEMENT SHALL PREVAIL.
//
// All rights created by patent grant or registration of a utility model or
// design patent are reserved.
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//
// The ConManDll contains the functional interface for
// access of the Cinterion HCxx Wireless Ethernet Adapters (USB).
// The following characteristics are supported:
//     - Detection/selection of the Cinterion HCxx Wireless Ethernet USB Adapters
//     - Activation/deactivation of the PIN verification, PIN verification
//     - Activation/deactivation of a packet data session
//     - Signal level indication
//
// Fundamental operation sequence
// ==============================
//
// 1.Step
// Register application
//     Call CmdApi_Register() to register a callback function.
//     The callback function is called by the DLL-context to
//     indicate messages (responses or indications).
//
// 2. Step
// Wait for available devices
//     If USB device (Cinterion HCxx Wireless Ethernet USB Adapter) exists, 
//     it is indicated by a message CMDAPI_MSG_DT_AVAILABLE.
//     Several devices can be available.
//
// 3. Step
// Establish device
//     Select a device (from the list of available devices)
//     and make a connection by a call of the function CmdApi_EstablishDevice().
//     The state of the connection is indicated by the following messages:
//         CMDAPI_MSG_DT_CONNECT - connection to the device successfully established
//         CMDAPI_MSG_DT_CONNECT_FAIL - connection to the device failed 
//  
//     If the connection is successfully created, then we are in the operational mode.
//
// 4. Step
// Operational mode
//     Now it is possible to start a packet data session.
//     Beforehand the SIM-Pin states can be evaluated or the SIM Pin can be verified.
//
// 5. Step
// Termination
//     Close the date packet session (CmdApi_StopConnectionReq()).
//     Disconnect from the current USB device (CmdApi_TerminateDevice()).
//     Log out from the use of the DLL (CmdApi_Unregister()).
//
///////////////////////////////////////////////////////////////////////////////



#pragma once

#ifdef CONMANDLL_EXPORTS
#define CONMANDLL_API __declspec(dllexport)
#else
#define CONMANDLL_API __declspec(dllimport)
#endif

#ifdef __cplusplus
extern "C" {            /* Assume C declarations for C++ */
#endif  /* __cplusplus */


// ----------------------------------------------------------------------------
// Result Codes
// ----------------------------------------------------------------------------
static const unsigned short CMDAPI_RESULT_SUCCESS =     0x0000;
static const unsigned short CMDAPI_RESULT_FAILURE =     0x0001;

// ----------------------------------------------------------------------------
// Error Codes
// ----------------------------------------------------------------------------
typedef enum
{
  CMDAPI_ERR_NONE                      = 0,
  CMDAPI_ERR_MALFORMED_MSG             = 1,
  CMDAPI_ERR_NO_MEMORY                 = 2,
  CMDAPI_ERR_INTERNAL                  = 3,
  CMDAPI_ERR_ABORTED                   = 4,
  CMDAPI_ERR_CLIENT_IDS_EXHAUSTED      = 5,
  CMDAPI_ERR_UNABORTABLE_TRANSACTION   = 6,
  CMDAPI_ERR_INVALID_CLIENT_ID         = 7,
  CMDAPI_ERR_NO_THRESHOLDS             = 8,
  CMDAPI_ERR_INVALID_HANDLE            = 9,
  CMDAPI_ERR_INVALID_PROFILE           = 10,
  CMDAPI_ERR_INVALID_PINID             = 11,
  CMDAPI_ERR_INCORRECT_PIN             = 12,
  CMDAPI_ERR_NO_NETWORK_FOUND          = 13,
  CMDAPI_ERR_CALL_FAILED               = 14,
  CMDAPI_ERR_OUT_OF_CALL               = 15,
  CMDAPI_ERR_NOT_PROVISIONED           = 16,  
  CMDAPI_ERR_MISSING_ARG               = 17,
  CMDAPI_ERR_ARG_TOO_LONG              = 19,
  CMDAPI_ERR_INVALID_TX_ID             = 22,
  CMDAPI_ERR_DEVICE_IN_USE             = 23,
  CMDAPI_ERR_OP_NETWORK_UNSUPPORTED    = 24,
  CMDAPI_ERR_OP_DEVICE_UNSUPPORTED     = 25,
  CMDAPI_ERR_NO_EFFECT                 = 26,
  CMDAPI_ERR_NO_FREE_PROFILE           = 27,
  CMDAPI_ERR_INVALID_PDP_TYPE          = 28,
  CMDAPI_ERR_INVALID_TECH_PREF         = 29,
  CMDAPI_ERR_INVALID_PROFILE_TYPE      = 30,
  CMDAPI_ERR_INVALID_SERVICE_TYPE      = 31,
  CMDAPI_ERR_INVALID_REGISTER_ACTION   = 32,
  CMDAPI_ERR_INVALID_PS_ATTACH_ACTION  = 33,
  CMDAPI_ERR_AUTHENTICATION_FAILED     = 34,
  CMDAPI_ERR_PIN_BLOCKED               = 35,
  CMDAPI_ERR_PIN_PERM_BLOCKED          = 36,
  CMDAPI_ERR_SIM_NOT_INITIALIZED       = 37,
  CMDAPI_ERR_MAX_QOS_REQUESTS_IN_USE   = 38,
  CMDAPI_ERR_INCORRECT_FLOW_FILTER     = 39,
  CMDAPI_ERR_NETWORK_QOS_UNAWARE       = 40,
  CMDAPI_ERR_INVALID_QOS_ID            = 41,
  CMDAPI_ERR_NUM_QOS_IDS               = 42,
  CMDAPI_ERR_QOS_UNAVAILABLE           = 43,
  CMDAPI_ERR_FLOW_SUSPENDED            = 44,
  CMDAPI_ERR_INVALID_DATA_FORMAT       = 45,
  CMDAPI_ERR_MAX,
  CMDAPI_ERR_WIDTH                     = 0xFFFF
};


// ----------------------------------------------------------------------------
// CmdApi Message Types
// ----------------------------------------------------------------------------
enum enum_CMDAPI_MSG_TYPE {
    CMDAPI_MSG_DT_AVAILABLE = 0,
    CMDAPI_MSG_DT_CONNECT,
    CMDAPI_MSG_DT_CONNECT_FAIL,
    CMDAPI_MSG_DT_DISCONNECT,
    CMDAPI_MSG_DT_DETACH,

    CMDAPI_MSG_MT_START_CONNECTION_RSP,
    CMDAPI_MSG_MT_ABORT_CONNECTION_RSP,
    CMDAPI_MSG_MT_STOP_CONNECTION_RSP,
    CMDAPI_MSG_MT_STATUS_CONNECTION_IND,

    CMDAPI_MSG_MT_SIGNAL_STRENGTH_RSP,
    CMDAPI_MSG_MT_SET_EVENT_REPORT_RSP,
    CMDAPI_MSG_MT_EVENT_REPORT_IND,

    CMDAPI_MSG_MT_GET_PIN_STATUS_RSP,
    CMDAPI_MSG_MT_SET_PIN_PROTECTION_RSP,
    CMDAPI_MSG_MT_VERIFY_PIN_RSP,

    CMDAPI_MSG_MT_SERVING_SYSTEM_RSP,
    CMDAPI_MSG_MT_SERVING_SYSTEM_IND,

    CMDAPI_MSG_UNKNOWN
};

// ----------------------------------------------------------------------------
// CmdApi Message Classes
// ----------------------------------------------------------------------------
enum enum_CMDAPI_MSG_CLASS {
    CMDAPI_MSGCLASS_NT_MESSAGE = 0,
    CMDAPI_MSGCLASS_NT_DEVICE,
    CMDAPI_MSGCLASS_UNKNOWN
};

// ----------------------------------------------------------------------------
// Message Structures
// ----------------------------------------------------------------------------

// ----------------------------------------------------------------------------
// CMDAPI_MSG_DT_AVAILABLE
//   update list of available network devices
// CMDAPI_MSG_DT_CONNECT
//   connection to device established
//   response to CmdApi_EstablishDevice()
// CMDAPI_MSG_DT_CONNECT_FAIL
//   conneciton to device failed
//   response to CmdApi_EstablishDevice()
// CMDAPI_MSG_DT_DISCONNECT
//   connection to device finished
//   response to CmdApi_TerminateDevice()
// CMDAPI_MSG_DT_DETACH
//   device detached from system
//   update list of the available devices
//   
//
// Parameters
// pList        : Pointer to zero terminated string list or NULL
//                list of network device names for establishing a packet data session or
//                handling of the PIN related functionality.
//                The end of the list contains an empty sting (two zero bytes)
// pReason      : Pointer to zero terminated string
//                Contains message specific text/information
//
// ----------------------------------------------------------------------------
typedef struct _msg_device {
    unsigned char       *pList;
    unsigned char       *pReason;
} msg_dt_device_t, *pmsg_dt_device_t;


// ----------------------------------------------------------------------------
// CMDAPI_MSG_MT_START_CONNECTION_RSP
// see CmdApi_StartConnectionReq()
//
// Parameters
// Result                   : CMDAPI_RESULT_SUCCESS
//                            CMDAPI_RESULT_FAILURE
//                              see error code 
//
// Error                    : CMDAPI_ERR_NONE 
//                              No error in request
//                            CMDAPI_ERR_INTERNAL 
//                              Unexpected error occurred during processing
//                            CMDAPI_ERR_MALFORMED_MSG 
//                              Message was not formulated correctly by the DLL or the
//                              message was corrupted during transmission
//                            CMDAPI_ERR_NO_MEMORY 
//                              Device could not allocate memory to formulate response
//                            CMDAPI_ERR_ARG_TOO_LONG 
//                              Argument passed in DLL larger than the available storage in
//                              the device
//                            CMDAPI_ERR_NO_NETWORK_FOUND 
//                              Connection could not be established with the serving network
//                            CMDAPI_ERR_IN_CALL START_NETWORK_INTERFACE 
//                              command has already been
//                              issued by the application
//                            CMDAPI_ERR_RADIO_IN_USE 
//                              Radio is not available to connect a packet data session on this
//                              network device
//                            CMDAPI_ERR_AUTHENTICATION_FAILED 
//                              Authentication credentials provided were incorrect
//                            CMDAPI_ERR_NO_EFFECT 
//                              DLL has already started the network interface
//
// DataHandle               : Handle identifying the call instance
//                            providing packet service
//                            The packet data handle must be
//                            retained by application and
//                            specified in the CmdApi_StopConnectionReq()
//                            message issued when the application
//                            is finished with the packet data
//                            session
//
// ----------------------------------------------------------------------------
typedef struct _msg_mt_start_connection_rsp {
    unsigned int        DataHandle;
    unsigned int        Result;
    unsigned int        Error;
} msg_mt_start_connection_rsp_t, *pmsg_mt_start_connection_rsp_t;

// ----------------------------------------------------------------------------
// CMDAPI_MSG_MT_ABORT_CONNECTION_RSP
// see CmdApi_AbortConnectionReq()
//
// Parameters
// Result                   : CMDAPI_RESULT_SUCCESS
//                            CMDAPI_RESULT_FAILURE
//                              see error code 
//
// Error                    : CMDAPI_ERR_NONE 
//                              No error in request
//                            CMDAPI_ERR_INTERNAL 
//                              Unexpected error occurred during processing
//                            CMDAPI_ERR_MALFORMED_MSG 
//                              Message was not formulated correctly by the DLL or the message
//                              was corrupted during transmission
//                            CMDAPI_ERR_INVALID_TX_ID
//                              TX_ID supplied in the request does not match pending CmdApi_StartConnectionReq,
//                              i.e., either the CmdApi_StartConnectionReq was not received or
//                              it has already been executed by the device
//
// ----------------------------------------------------------------------------
typedef struct _msg_mt_abort_connection_rsp {
    unsigned int        Result;
    unsigned int        Error;
} msg_mt_abort_connection_rsp_t, *pmsg_mt_abort_connection_rsp_t;

// ----------------------------------------------------------------------------
// CMDAPI_MSG_MT_STOP_CONNECTION_RSP
// see CmdApi_StopConnectionReq()
//
// Parameters
// Result                   : CMDAPI_RESULT_SUCCESS
//                            CMDAPI_RESULT_FAILURE
//                              see error code 
//
// Error                    : CMDAPI_ERR_NONE 
//                              No error in request
//                            CMDAPI_ERR_INTERNAL 
//                              Unexpected error occurred during processing
//                            CMDAPI_ERR_MALFORMED_MSG 
//                              Message was not formulated correctly by the DLL or the message
//                              was corrupted during transmission
//                            CMDAPI_ERR_INVALID_HANDLE 
//                              DataHandle provided in the CMDAPI_MSG_MT_START_CONNECTION_RSP
//                              is not valid, i.e., is not assigned to the device
//
// ----------------------------------------------------------------------------
typedef struct _msg_mt_stop_connection_rsp {
    unsigned int        Result;
    unsigned int        Error;
} msg_mt_stop_connection_rsp_t, *pmsg_mt_stop_connection_rsp_t;


//-----------------------------------------------------------------------------
// CMDAPI_MSG_MT_STATUS_CONNECTION_IND
// This message indicates a change in the current packet data connection status
//
// Parameters
// Connection_Status        : Current link status
//                            STATUS_PKT_DATA_DISCONNECTED
//                            STATUS_PKT_DATA_CONNECTED
//                              Indicates whether a network connection
//                              has been established
//                            STATUS_PKT_DATA_SUSPENDED
//
//                            STATUS_PKT_DATA_AUTHENTICATING
//                              Value of authenticating indicates 
//                              authentication started, but not connected
//
//-----------------------------------------------------------------------------
typedef enum {
    STATUS_PKT_DATA_DISCONNECTED    = 1,
    STATUS_PKT_DATA_CONNECTED       = 2,
    STATUS_PKT_DATA_SUSPENDED       = 3,
    STATUS_PKT_DATA_AUTHENTICATING  = 4
};

typedef struct _msg_mt_status_connection_ind {
    unsigned int        Connection_Status;
} msg_mt_status_connection_int_t, *pmsg_mt_status_connection_ind_t;

//-----------------------------------------------------------------------------
// CMDAPI_MSG_MT_SIGNAL_STRENGTH_RSP
// see CmdApi_GetSignalStrengthReq()
//
// Parameters
// Result                   : CMDAPI_RESULT_SUCCESS
//                            CMDAPI_RESULT_FAILURE
//                              see error code 
//
// Error                    : CMDAPI_ERR_NONE 
//                              No error in request
//                            CMDAPI_ERR_INTERNAL
//                              An unexpected error occurred during processing
//                            CMDAPI_ERR_MALFORMED_MSG
//                              Message was not formulated correctly by the DLL or 
//                              message was corrupted during transmission
//                            CMDAPI_ERR_NO_MEMORY
//                              Device could not allocate memory to formulate response
// 
// SignalStrength           : Received signal strength in dBm
//                            for UMTS, this indicates forward link pilot Ec; 
//                            for GSM, this indicates the received signal strength
//
// RadioIf                  : The radio interface technology of the signal being measured:
//                              0x00  None (no service) 
//                              0x01  cdma2000 1X (not supported)
//                              0x02  cdma2000 HRPD (1xEV-DO) (not supported)
//                              0x03 - AMPS (not supported)
//                              0x04  GSM
//                              0x05  UMTS
//
//-----------------------------------------------------------------------------
typedef struct _msg_mt_signal_strength_rsp {
    unsigned int        Result;
    unsigned int        Error;
    signed char         SignalStrength;
    unsigned char       RadioIf;
} msg_mt_signal_strength_rsp_t, *pmsg_mt_signal_strength_rsp_t;


//-----------------------------------------------------------------------------
// CMDAPI_MSG_MT_SET_EVENT_REPORT_RSP
// see CmdApi_SetEventReportReq()
//
// Parameters
// Result                   : CMDAPI_RESULT_SUCCESS
//                            CMDAPI_RESULT_FAILURE
//                              see error code 
//
// Error                    : CMDAPI_ERR_NONE 
//                              No error in request
//                            CMDAPI_ERR_INTERNAL
//                              An unexpected error occurred during processing
//                            CMDAPI_ERR_MALFORMED_MSG
//                              Message was not formulated correctly by the DLL or 
//                              message was corrupted during transmission
//                            CMDAPI_ERR_ARG_TOO_LONG
//                              More than the maximum allowed thresholds were specified
//
//-----------------------------------------------------------------------------
typedef struct _msg_mt_set_event_report_rsp {
    unsigned int        Result;
    unsigned int        Error;
} msg_mt_set_event_report_rsp_t, *pmsg_mt_set_event_report_rsp_t;

//-----------------------------------------------------------------------------
// CMDAPI_MSG_MT_EVENT_REPORT_IND
// see CmdApi_SetEventReportReq()
//
// Parameters
// SignalStrength           : Received signal strength in dBm
//                            for UMTS, this indicates forward link pilot Ec; 
//                            for GSM, this indicates the received signal strength
//
// RadioIf                  : The radio interface technology of the signal being measured:
//                              0x00  None (no service) 
//                              0x01  cdma2000 1X (not supported)
//                              0x02  cdma2000 HRPD (1xEV-DO) (not supported)
//                              0x03 - AMPS (not supported)
//                              0x04  GSM
//                              0x05  UMTS
//
//-----------------------------------------------------------------------------
typedef struct _msg_mt_event_report_ind {
    signed char         SignalStrength;
    unsigned char       RadioIf;
} msg_mt_event_report_ind_t, *pmsg_mt_event_report_ind_t;


//-----------------------------------------------------------------------------
// CMDAPI_MSG_MT_SERVING_SYSTEM_RSP
// see CmdApi_GetServingSystemReq()
//
// Parameters
// Result                   : CMDAPI_RESULT_SUCCESS
//                            CMDAPI_RESULT_FAILURE
//                              see error code 
//
// Error                    : CMDAPI_ERR_NONE 
//                              No error in request
//                            CMDAPI_ERR_INTERNAL
//                              An unexpected error occurred during processing
//                            CMDAPI_ERR_MALFORMED_MSG
//                              Message was not formulated correctly by the DLL or 
//                              message was corrupted during transmission
//                            CMDAPI_ERR_NO_MEMORY
//                              Device could not allocate memory to formulate response
//
// RegistrationState        : The registration state of the device
//                              0  Not registered, device is not currently 
//                                  searching for a new network to provide service
//                              1  Registered with a network
//                              2  Not registered, but mobile is currently 
//                                  searching for a new network to provide service
//                              3  Registration denied by the visible network
//                              4  Registration state is unknown
//
// CSAttachState            : The circuit switch domain attach state of the device
//                              0  Unknown or not applicable
//                              1  Attached
//                              2  Detached  
//
// PSAttachState            : The packet domain attach state of the device  
//                              0  Unknown or not applicable
//                              1  Attached
//                              2  Detached
//
// RegisteredNetwork        : Type of radio access network which 
//                            the device is registered on:
//                              0x00  Unknown
//                              0x01  3GPP2 network
//                              0x02  3GPP network
//
// NumRadioIfs              : Number of radio interfaces currently in use; 
//                            indicates how many radio interface identifiers follow this field
//
// pRadioIfslist            : Pointer to list of the Radio interfaces currently in use:
//                              0x01  cdma2000 1X (not supported)
//                              0x02  cdma2000 HRPD (1xEV-DO) (not supported)
//                              0x03  AMPS (not supported)
//                              0x04  GSM
//                              0x05  UMTS
// 
//-----------------------------------------------------------------------------
typedef struct _msg_mt_serving_system_rsp {
    unsigned int        Result;
    unsigned int        Error;

    unsigned char       RegistrationState;
    unsigned char       CSAttachState;
    unsigned char       PSAttachState;
    unsigned char       RegisteredNetwork;
    unsigned char       NumRadioIfs;
    unsigned char       *pRadioIfslist;

} msg_mt_serving_system_rsp_t, *pmsg_mt_serving_system_rsp_t;

//-----------------------------------------------------------------------------
// CMDAPI_MSG_MT_SERVING_SYSTEM_IND
// This indicates a change in the current serving system registration 
// state and/or radio technology
//
// Parameters
// RegistrationState        : The registration state of the device
//                              0  Not registered, device is not currently 
//                                  searching for a new network to provide service
//                              1  Registered with a network
//                              2  Not registered, but mobile is currently 
//                                  searching for a new network to provide service
//                              3  Registration denied by the visible network
//                              4  Registration state is unknown
//
// CSAttachState            : The circuit switch domain attach state of the device
//                              0  Unknown or not applicable
//                              1  Attached
//                              2  Detached  
//
// PSAttachState            : The packet domain attach state of the device  
//                              0  Unknown or not applicable
//                              1  Attached
//                              2  Detached
//
// RegisteredNetwork        : Type of radio access network which 
//                            the device is registered on:
//                              0x00  Unknown
//                              0x01  3GPP2 network
//                              0x02  3GPP network
//
// NumRadioIfs              : Number of radio interfaces currently in use; 
//                            indicates how many radio interface identifiers follow this field
//
// pRadioIfslist            : Pointer to list of the Radio interfaces currently in use:
//                              0x01  cdma2000 1X (not supported)
//                              0x02  cdma2000 HRPD (1xEV-DO) (not supported)
//                              0x03  AMPS (not supported)
//                              0x04  GSM
//                              0x05  UMTS
// 
//-----------------------------------------------------------------------------
typedef struct _msg_mt_serving_system_ind {
    unsigned char       RegistrationState;
    unsigned char       CSAttachState;
    unsigned char       PSAttachState;
    unsigned char       RegisteredNetwork;
    unsigned char       NumRadioIfs;
    unsigned char       *pRadioIfslist;
} msg_mt_serving_system_ind_t, *pmsg_mt_serving_system_ind_t;

//-----------------------------------------------------------------------------
// CMDAPI_MSG_MT_GET_PIN_STATUS_RSP
// see CmdApi_GetPinStatusReq()
//
//
// Parameters   
// Result                   : CMDAPI_RESULT_SUCCESS
//                            CMDAPI_RESULT_FAILURE
//                              see error code 
//
// Error                    : CMDAPI_ERR_NONE
//                              Indicates that there was no error in the request
//                            CMDAPI_ERR_INTERNAL
//                              Indicates that an unexpected error occurred during processing
//                            CMDAPI_ERR_MALFORMED_MSG
//                              Indicates that the message was not formulated correctly by the DLL
//                              or the message was corrupted during transmission
//                            CMDAPI_ERR_NO_MEMORY
//                              Indicates that the device could not allocate memory to formulate the response
//                            CMDAPI_ERR_SIM_NOT_INITIALIZED
//                              Indicates that the PIN is not yet initialized  
//                              because SIM initialization has not finished; try the PIN operation later
//                            CMDAPI_ERR_OP_DEVICE_UNSUPPORTED
//                              Indicates that the operation is not supported by the device
//
// IsPin1Status             : pin1 status values are set 
//                            (see params Pin1Status, Pin1VerifyRetriesLeft, Pin1UnblockRetriesLeft)

// IsPin2Status             : pin2 status values are set 
//                            (see params Pin2Status, Pin2VerifyRetriesLeft, Pin2UnblockRetriesLeft)
//
// Pin1Status               : PIN_NOT_INITIALIZED
//                            PIN_ENABLED_NOT_VERIFIED    
//                            PIN_ENABLED_VERIFIED        
//                            PIN_DISABLED                
//                            PIN_BLOCKED                 
//                            PIN_PERMANENTLY_BLOCKED     
//                            PIN_UNBLOCKED               
//                            PIN_CHANGED                
//
// Pin1VerifyRetriesLeft    : Indicates the number of retries left before the pin will be blocked
//
// Pin1UnblockRetriesLeft   : Indicates the number of unblock retries left before
//                            the PIN will be permanently blocked, i.e., SIM is unusable
//
// Pin2Status               : PIN_NOT_INITIALIZED
//                            PIN_ENABLED_NOT_VERIFIED    
//                            PIN_ENABLED_VERIFIED        
//                            PIN_DISABLED                
//                            PIN_BLOCKED                 
//                            PIN_PERMANENTLY_BLOCKED     
//                            PIN_UNBLOCKED               
//                            PIN_CHANGED                
//
// Pin2VerifyRetriesLeft    : Indicates the number of retries left before the pin will be blocked
//
// Pin2UnblockRetriesLeft   : Indicates the number of unblock retries left before
//                            the PIN will be permanently blocked, i.e., SIM is unusable
//-----------------------------------------------------------------------------
typedef enum {
    PIN_NOT_INITIALIZED         = 0,
    PIN_ENABLED_NOT_VERIFIED    = 1,
    PIN_ENABLED_VERIFIED        = 2,
    PIN_DISABLED                = 3,
    PIN_BLOCKED                 = 4,
    PIN_PERMANENTLY_BLOCKED     = 5,
    PIN_UNBLOCKED               = 6,
    PIN_CHANGED                 = 7
};

typedef struct _msg_mt_get_pin_status_rsp {
    unsigned int        Result;
    unsigned int        Error;

    bool                IsPin1Status;
    bool                IsPin2Status;

    unsigned char       Pin1Status;
    unsigned char       Pin1VerifyRetriesLeft;
    unsigned char       Pin1UnblockRetriesLeft;

    unsigned char       Pin2Status;
    unsigned char       Pin2VerifyRetriesLeft;
    unsigned char       Pin2UnblockRetriesLeft;
} msg_mt_get_pin_status_rsp_t, *pmsg_mt_get_pin_status_rsp_t;

//-----------------------------------------------------------------------------
// CMDAPI_MSG_MT_CHANGE_PIN_RSP
// see CmdApi_ChangePinReq()
//
//
// Parameters
// Result                   : CMDAPI_RESULT_SUCCESS
//                            CMDAPI_RESULT_FAILURE
//                              see Error code 
//
// Error                    : CMDAPI_ERR_NONE
//                              Indicates that there was no error in the request
//                            CMDAPI_ERR_INTERNAL
//                              Indicates that an unexpected error occurred during processing
//                            CMDAPI_ERR_MALFORMED_MSG
//                              Indicates that the message was not formulated correctly by the DLL
//                              or the message was corrupted during transmission
//                            CMDAPI_ERR_NO_MEMORY
//                              Indicates that the device could not allocate memory to formulate the response
//                            CMDAPI_ERR_NO_EFFECT
//                              Indicates that the operation had no effect
//                            CMDAPI_ERR_ARG_TOO_LONG
//                              Indicates that the device cannot handle the length of the PIN given in the request
//                            CMDAPI_ERR_INCORRECT_PIN
//                              Indicates that the PIN given in the request is incorrect
//                            CMDAPI_ERR_PIN_BLOCKED
//                              Indicates that the PIN is blocked; an unblock operation needs to be issued
//                            CMDAPI_ERR_PIN_PERM_BLOCKED
//                              Indicates that the PIN is permanently blocked; SIM is unusable
//                            CMDAPI_ERR_SIM_NOT_INITIALIZED
//                              Indicates that the PIN is not yet initialized 
//                              because SIM initialization has not finished; try the PIN operation later
//                            CMDAPI_ERR_OP_DEVICE_UNSUPPORTED
//                              Indicates that the operation is not supported by the device
//                            CMDAPI_ERR_MISSING_ARG
//                              Indicates that some TLV was missing in the request
//                            CMDAPI_ERR_INVALID_PINID
//                              Indicates that the PIN given in the request is invalid
//
// IsRetriesLeft            : following parameters are valid
//
// VerifyRetriesLeft        : Indicates the number of retries left before the PIN will be blocked
// UnblockRetriesLeft       : Indicates the number of unblock retries left before
//                            the PIN will be permanently blocked, i.e., SIM is unusable
//
//-----------------------------------------------------------------------------
typedef struct _msg_mt_change_pin_rsp {
    unsigned int        Result;
    unsigned int        Error;
    
    bool                IsRetriesLeft;
    unsigned char       VerifyRetriesLeft;
    unsigned char       UnblockRetriesLeft;
} msg_mt_change_pin_rsp_t, *pmsg_mt_change_pin_rsp_t;

//-----------------------------------------------------------------------------
// CMDAPI_MSG_MT_SET_PIN_PROTECTION_RSP
// see CmdApi_SetPinProtectionReq()
//
// Parameters
// Result                   : CMDAPI_RESULT_SUCCESS
//                            CMDAPI_RESULT_FAILURE
//                              see Error code 
//
// Error                    : CMDAPI_ERR_NONE
//                              No error in the request
//                            CMDAPI_ERR_INTERNAL
//                              Indicates an unexpected error occurred during processing
//                            CMDAPI_ERR_MALFORMED_MSG
//                              Indicates that the message was not formulated correctly by the DLL 
//                              or the message was corrupted during transmission
//                            CMDAPI_ERR_NO_MEMORY
//                              Indicates that the device could not allocate memory to formulate the response
//                            CMDAPI_ERR_NO_EFFECT
//                              Indicates that the operation had no effect
//                            CMDAPI_ERR_ARG_TOO_LONG
//                              Indicates that the device cannot handle the length of the PIN given
//                            CMDAPI_ERR_INCORRECT_PIN
//                              Indicates that the PIN given in the request is incorrect
//                            CMDAPI_ERR_PIN_BLOCKED
//                              Indicates that the PIN is blocked; an unblock operation needs to be issued
//                            CMDAPI_ERR_PIN_PERM_BLOCKED
//                              Indicates that the PIN is permanently blocked; the SIM is unusable
//                            CMDAPI_ERR_SIM_NOT_INITIALIZED
//                              Indicates that the PIN is not yet initialized 
//                              because the SIM initialization has not finished; try the PIN operation later
//                            CMDAPI_ERR_OP_DEVICE_UNSUPPORTED
//                              Indicates that the operation is not supported by the device
//                            CMDAPI_ERR_MISSING_ARG
//                              Indicates that arguments were missing in the request from DLL
//                            CMDAPI_ERR_INVALID_PINID
//                              Indicates that the PIN given in the request is invalid
//
// IsRetriesLeft            : following parameters are valid
//
// VerifyRetriesLeft        : Indicates the number of retries left before the PIN will be blocked
// UnblockRetriesLeft       : Indicates the number of unblock retries left before
//                            the PIN will be permanently blocked, i.e., SIM is unusable
//
//-----------------------------------------------------------------------------
typedef struct _msg_mt_set_pin_protection_rsp {
    unsigned int        Result;
    unsigned int        Error;

    bool                IsRetriesLeft;
    unsigned char       VerifyRetriesLeft;
    unsigned char       UnblockRetriesLeft;
} msg_mt_set_pin_protection_rsp_t, *pmsg_mt_set_pin_protection_rsp_t;

//-----------------------------------------------------------------------------
// CMDAPI_MSG_MT_UNBLOCK_PIN_RSP
// see CmdApi_UnblockPinReq()
//
// Parameters
// Result                   : CMDAPI_RESULT_SUCCESS
//                            CMDAPI_RESULT_FAILURE
//                              see Error code 
//
// Error                    : CMDAPI_ERR_NONE
//                              No error in the request
//                            CMDAPI_ERR_INTERNAL
//                              Indicates an unexpected error occurred during processing
//                            CMDAPI_ERR_MALFORMED_MSG
//                              Indicates that the message was not formulated correctly by the DLL 
//                              or the message was corrupted during transmission
//                            CMDAPI_ERR_NO_MEMORY
//                              Indicates that the device could not allocate memory to formulate the response
//                            CMDAPI_ERR_NO_EFFECT
//                              Indicates that the operation had no effect
//                            CMDAPI_ERR_ARG_TOO_LONG
//                              Indicates that the device cannot handle the length of the PIN given
//                            CMDAPI_ERR_INCORRECT_PIN
//                              Indicates that the PIN or PUK given in the request is incorrect
//                            CMDAPI_ERR_PIN_BLOCKED
//                              Indicates that the PIN is blocked; an unblock operation needs to be issued
//                            CMDAPI_ERR_PIN_PERM_BLOCKED
//                              Indicates that the PIN is permanently blocked; the SIM is unusable
//                            CMDAPI_ERR_SIM_NOT_INITIALIZED
//                              Indicates that the PIN is not yet initialized 
//                              because the SIM initialization has not finished; try the PIN operation later
//                            CMDAPI_ERR_OP_DEVICE_UNSUPPORTED
//                              Indicates that the operation is not supported by the device
//                            CMDAPI_ERR_MISSING_ARG
//                              Indicates that arguments were missing in the request from DLL
//                            CMDAPI_ERR_INVALID_PINID
//                              Indicates that the PIN given in the request is invalid
//
// IsRetriesLeft            : following parameters are valid
//
// VerifyRetriesLeft        : Indicates the number of retries left after which the PIN will be blocked
// UnblockRetriesLeft       : Indicates the number of unblock retries left after which 
//                            the PIN will be permanently blocked, i.e., SIM is unusable
//
//-----------------------------------------------------------------------------
typedef struct _msg_mt_unblock_pin_rsp {
    unsigned int        Result;
    unsigned int        Error;

    bool                IsRetriesLeft;
    unsigned char       VerifyRetriesLeft;
    unsigned char       UnblockRetriesLeft;
} msg_mt_unblock_pin_rsp_t, *pmsg_mt_unblock_pin_rsp_t;

//-----------------------------------------------------------------------------
// CMDAPI_MSG_MT_VERIFY_PIN_RSP
// see CmdApi_VerifyPinReq()
//
// Parameters
// Result                   : CMDAPI_RESULT_SUCCESS
//                            CMDAPI_RESULT_FAILURE
//                              see Error code 
//
// Error                    : CMDAPI_ERR_NONE
//                              No error in the request
//                            CMDAPI_ERR_INTERNAL
//                              Indicates an unexpected error occurred during processing
//                            CMDAPI_ERR_MALFORMED_MSG
//                              Indicates that the message was not formulated correctly by the DLL 
//                              or the message was corrupted during transmission
//                            CMDAPI_ERR_NO_MEMORY
//                              Indicates that the device could not allocate memory to formulate the response
//                            CMDAPI_ERR_NO_EFFECT
//                              Indicates that the operation had no effect
//                            CMDAPI_ERR_ARG_TOO_LONG
//                              Indicates that the device cannot handle the length of the PIN given
//                            CMDAPI_ERR_INCORRECT_PIN
//                              Indicates that the PIN or PUK given in the request is incorrect
//                            CMDAPI_ERR_PIN_BLOCKED
//                              Indicates that the PIN is blocked; an unblock operation needs to be issued
//                            CMDAPI_ERR_PIN_PERM_BLOCKED
//                              Indicates that the PIN is permanently blocked; the SIM is unusable
//                            CMDAPI_ERR_SIM_NOT_INITIALIZED
//                              Indicates that the PIN is not yet initialized 
//                              because the SIM initialization has not finished; try the PIN operation later
//                            CMDAPI_ERR_OP_DEVICE_UNSUPPORTED
//                              Indicates that the operation is not supported by the device
//                            CMDAPI_ERR_MISSING_ARG
//                              Indicates that arguments were missing in the request from DLL
//                            CMDAPI_ERR_INVALID_PINID
//                              Indicates that the PIN given in the request is invalid
//
// IsRetriesLeft            : following parameters are valid
//
// VerifyRetriesLeft        : Indicates the number of retries left before the PIN will be blocked
// UnblockRetriesLeft       : Indicates the number of unblock retries left before
//                            the PIN will be permanently blocked, i.e., SIM is unusable
//
//-----------------------------------------------------------------------------
typedef struct _msg_mt_verify_pin_rsp {
    unsigned int        Result;
    unsigned int        Error;

    bool                IsRetriesLeft;
    unsigned char       VerifyRetriesLeft;
    unsigned char       UnblockRetriesLeft;

} msg_mt_verify_pin_rsp_t, *pmsg_mt_verify_pin_rsp_t;


// CmdApi Message Classes
typedef struct _msg {
    unsigned int    msg_class;
    unsigned int    msg_type;
    
    union _msg_union {
        msg_dt_device_t                 msg_dt_device;

        msg_mt_start_connection_rsp_t   msg_mt_start_connection_rsp;
        msg_mt_abort_connection_rsp_t   msg_mt_abort_connection_rsp;
        msg_mt_stop_connection_rsp_t    msg_mt_stop_connection_rsp;
        msg_mt_status_connection_int_t  msg_mt_status_connection_ind;

        msg_mt_signal_strength_rsp_t    msg_mt_signal_strength_rsp;
        msg_mt_set_event_report_rsp_t   msg_mt_set_event_report_rsp;
        msg_mt_event_report_ind_t       msg_mt_event_report_ind;

        msg_mt_get_pin_status_rsp_t     msg_mt_get_pin_status_rsp;
        msg_mt_change_pin_rsp_t         msg_mt_change_pin_rsp;
        msg_mt_set_pin_protection_rsp_t msg_mt_set_pin_protection_rsp;
        msg_mt_unblock_pin_rsp_t        msg_mt_unblock_pin_rsp;
        msg_mt_verify_pin_rsp_t         msg_mt_verify_pin_rsp;

        msg_mt_serving_system_rsp_t     msg_mt_serving_system_rsp;
        msg_mt_serving_system_ind_t     msg_mt_serving_system_ind;

    } msg;


} CMD_API_MSG_T, *PCMD_API_MSG_T;



//-----------------------------------------------------------------------------
// CmdApi_Register
//
// Register callback function for message handling.
// The callback function is called by the DLL-context to
// handle responses or indications.
// The call of CmdApi_Register() starts the message management of the DLL.
//
// Parameters
// pFn      : Pointer to callback function 
//            (see also type declaration PCMDAPI_MSG_CALLBACK)
//
// Return   
// true     : success
// false    : callback function is already registered
//-----------------------------------------------------------------------------
typedef void (*PCMDAPI_MSG_CALLBACK)(PCMD_API_MSG_T pCmdApiMsg);
CONMANDLL_API bool CmdApi_Register
(
    PCMDAPI_MSG_CALLBACK pFn
);

//-----------------------------------------------------------------------------
// CmdApi_Unregister
//
// Deregister callback function for message handling.
// The call of CmdApi_Unregister() stops the message management of the DLL.
//
// Parameters
// none
//
// Return   
// true     : success
//-----------------------------------------------------------------------------
CONMANDLL_API bool CmdApi_Unregister
(
    void
);

//-----------------------------------------------------------------------------
// CmdApi_EstablishDevice
//
// Open a connection to a Cinterion USB NDIS device (e.g. HC25, HC28, ...).
// The list of the message CMDAPI_MSG_DT_AVAILABLE contains the valid device names.
// If the connection was successfully established, the response CMDAPI_MSG_DT_CONNECT is sent.
// In case of an error the response CMDAPI_MSG_DT_CONNECT_FAIL is sent.
//
// Parameters
// pDeviceName  : Pointer to zero terminated string of network device name
//
// Return   
// true         : success
// false        : - callback function not registered (see CmdApi_Register())
//                - string of the network device name too long or not zero terminated
//
//-----------------------------------------------------------------------------
CONMANDLL_API bool CmdApi_EstablishDevice
(
    unsigned char *pDeviceName 
);

//-----------------------------------------------------------------------------
// CmdApi_TerminateDevice
// Close a connection to a Cinterion USB NDIS device (e.g. HC25, HC28, ...).
// The response messsage CMDAPI_MSG_DT_DISCONNECT is sent.
//
// Parameters
// none
//
// Return   
// true         : success
// false        : - callback function not registered (see CmdApi_Register())
//                - string of the network device name too long or not zero terminated
//
//-----------------------------------------------------------------------------
CONMANDLL_API bool CmdApi_TerminateDevice
(
    void
);

//-----------------------------------------------------------------------------
// CmdApi_StartConnectionReq
//
// This function activates a packet data session (if not already started).
// It is only at the same time possible for one connection.
// (see Response Message CMDAPI_MSG_MT_START_CONNECTION_RSP)
//
// Parameters
// pPrimaryDnsPref      : Pointer to zero terminated string or NULL
//                        Primary DNS IPv4 Address Preference;
//                        The value may be used as a preference during
//                        negotiation with the network. If not specified, the
//                        wireless device will attempt to obtain the DNS
//                        address automatically from the network. The
//                        negotiated value is provided to the host via DHCP.
//                        (format of Sting: "xxx.xxx.xxx.xxx")
//
// pSecondaryDnsPref    : Pointer to zero terminated string or NULL
//                        Secondary DNS IPv4 Address Preference
//                        The value may be used as a preference during
//                        negotiation with the network. If not specified, the
//                        wireless device will attempt to obtain the DNS
//                        address automatically from the network. The
//                        negotiated value is provided to the host via DHCP.
//                        (format of Sting: "xxx.xxx.xxx.xxx")
//
// pPrimaryNbnsPref     : Pointer to zero terminated string or NULL
//                        Primary NetBIOS name server address;
//                        The specified IPv4 address is
//                        requested as the primary NBNS
//                        server during data session
//                        establishment. If not provided, the
//                        primary NBNS server address is
//                        obtained automatically from the
//                        network. The result of negotiation
//                        (the assigned address) is provided to
//                        the host via DHCP.
//                        (format of Sting: "xxx.xxx.xxx.xxx")
//
// pSecondaryNbnsPref   : Pointer to zero terminated string or NULL
//                        Secondary NetBIOS name server address;
//                        The specified IPv4 address is
//                        requested as the secondary NBNS
//                        server during data session
//                        establishment. If not provided, the
//                        secondary NBNS server address is
//                        obtained automatically from the
//                        network. The result of negotiation
//                        (the assigned address) is provided to
//                        the host via DHCP.
//                        (format of Sting: "xxx.xxx.xxx.xxx")
//
// pApnName             : Pointer to zero terminated string or NULL
//                        Access point name  A string parameter that is a
//                        logical name used to select the GGSN and the
//                        external packet data network
//
// pIpv4AddrPref        : Pointer to zero terminated string or NULL
//                        Preferred IPv4 address to be assigned to the TE
//                        The actual assigned address is negotiated with
//                        the network and may differ from this value. If not
//                        specified, the IPv4 Address is obtained
//                        automatically from the network. The assigned
//                        value is provided to the host via DHCP.
//
// AuthPref             : Value that indicates the authentication algorithm preference.
//                          Valid values:
//                              0 - AUTHPREF_AUTO     -> the device decides which 
//                                                       authentication procedure is performed 
//                                                       while setting up the data session
//                              1 - AUTHPREF_PAP      -> PAP preference         
//                              2 - AUTHPREF_CHAP     -> CHAP preference
//                              3 - AUTHPREF_PAP_CHAP -> the device decides which 
//                                                       authentication procedure is performed 
//                                                       while setting up the data session
//
// pUsername            : Pointer to zero terminated string or NULL
//                        Username to be used during data network
//                        authentication
//
// pPassword            : Pointer to zero terminated string or NULL
//                        Password to be used during data network
//                        authentication
//
// Return   
// 0                    : Error
//                        - callback function not registered (see CmdApi_Register())
//                        - one of the string values too long or not zero terminated
//                        - wrong AuthPref value
// > 0                  : ID for this Request;
//                        This value is required for an abort of this "Start Connection" Request
//                        (see CmdApi_AbortConnectionReq())
//                        
//-----------------------------------------------------------------------------
enum enum_AUTHPREF {
    AUTHPREF_AUTO = 0,
    AUTHPREF_PAP,        
    AUTHPREF_CHAP,    
    AUTHPREF_PAP_CHAP,
};

CONMANDLL_API unsigned int CmdApi_StartConnectionReq
(
    unsigned char   *pPrimaryDnsPref,
    unsigned char   *pSecondaryDnsPref,
    unsigned char   *pPrimaryNbnsPref,
    unsigned char   *pSecondaryNbnsPref,
    unsigned char   *pApnName,
    unsigned char   *pIpv4AddrPref,
    int             AuthPref,
    unsigned char   *pUsername,
    unsigned char   *pPassword
);

//-----------------------------------------------------------------------------
// CmdApi_AbortConnectionReq
//
// This function is called to abort a running Connection Request
// This command aborts a previously issued CmdApi_StartConnectionReq command. 
// It is useful for requests that take a long time to execute, 
// in the case where the user is no longer interested in the result.
// (see Response Message CMDAPI_MSG_MT_ABORT_CONNECTION_RSP)
//
// Parameters
// txId         : value, ID for the running Connection Request
//                (see Return from CmdApi_StartConnectionReq())
//
// Return   
// true         : success
// false        : - callback function not registered (see CmdApi_Register())
//
//-----------------------------------------------------------------------------
CONMANDLL_API bool CmdApi_AbortConnectionReq
(
    unsigned int    txId
);

//-----------------------------------------------------------------------------
// CmdApi_StopConnectionReq
// (see Response Message CMDAPI_MSG_MT_STOP_CONNECTION_RSP)
//
// This fuction deactivates a packet data session
//
// Parameters
// DataHandle   : value
//                Handle identifying the packet data session to
//                stop this connection.
//                This value must be the handle
//                previously returned by CMDAPI_MSG_MT_START_CONNECTION_RSP
//
// Return   
// true         : success
// false        : - callback function not registered (see CmdApi_Register())
//                
//-----------------------------------------------------------------------------
CONMANDLL_API bool CmdApi_StopConnectionReq
(
    unsigned int    DataHandle
);

//-----------------------------------------------------------------------------
// CmdApi_GetSignalStrengthReq
//
// This function queries the current signal strength as measured by the device.
// (see Response Message CMDAPI_MSG_MT_SIGNAL_STRENGTH_RSP)
//
// Parameters
// none
//
// Return   
// true         : success
// false        : - callback function not registered (see CmdApi_Register())
//
//-----------------------------------------------------------------------------
CONMANDLL_API bool CmdApi_GetSignalStrengthReq
(
    void
);

//-----------------------------------------------------------------------------
// CmdApi_SetEventReportReq
//
// This function sets the signal strength reporting conditions for the device.
// (see Response Message CMDAPI_MSG_MT_SET_EVENT_REPORT_RSP
//  and Indication CMDAPI_MSG_MT_EVENT_REPORT_IND)
//
// Parameters
// Report           : true  - Report
//                    false - Do not report
// NumRangeLimits   : The number of thresholds following a maximum of five thresholds 
//                    may be specified; 
//                    if report signal strength is set, at least one threshold must be specified.
// RangeLimits      : Pointer to a sequence of thresholds delimiting signal strength bands; 
//                    each threshold specifies the signal strength (in dBm) 
//                    at which an event report indication including the current signal strength
//                    will be sent to the application (see CMDAPI_MSG_MT_EVENT_REPORT_IND).
//                    The threshold is a signed, one byte value; 
//                    valid values are -128 dBm to +127 dBm.
//
// Return   
// true         : success
// false        : - callback function not registered (see CmdApi_Register())
//                - NumRangeLimits is not equal 1..5
//                - The Pointer RangeLimits is NULL
//
//-----------------------------------------------------------------------------
CONMANDLL_API bool CmdApi_SetEventReportReq
(
    bool            Report,
    unsigned int    NumRangeLimits,
    signed char    *RangeLimits
);

//-----------------------------------------------------------------------------
// CmdApi_GetServingSystemReq
//
// This function queries the current serving system and registration information, 
// including the system identification, registration state, and radio technology information.
// (see Response Message CMDAPI_MSG_MT_SERVING_SYSTEM_RSP)
//
// Parameters
// none
//
// Return   
// true         : success
// false        : - callback function not registered (see CmdApi_Register())
//
//-----------------------------------------------------------------------------
CONMANDLL_API bool CmdApi_GetServingSystemReq
(
    void
);

//-----------------------------------------------------------------------------
// CmdApi_GetPinStatusReq
// 
// This function is called to get the status of the pin (PIN1, PIN2)
// (see Response Message CMDAPI_MSG_MT_GET_PIN_STATUS_RSP)
//
// Parameters
// none
//
// Return   
// true         : success
// false        : - callback function not registered (see CmdApi_Register())
//                
//-----------------------------------------------------------------------------
CONMANDLL_API bool CmdApi_GetPinStatusReq
(
    void
);

//-----------------------------------------------------------------------------
// CmdApi_SetPinProtectionReq
//
// This function enables or disables protection of SIM contents by a given PIN
// (PIN1 or PIN2).
// (see Response Message CMDAPI_MSG_MT_SET_PIN_PROTECTION_RSP)
//
// Parameters
// PinId             : Specifies the ID of the PIN to be enabled or disabled 
//                     (valid values 1 = PIN1, 2 = PIN2)
// ProtectionSetting : Specifies whether the PIN is enabled
//                     valid values are:
//                       TRUE  Enable PIN
//                       FALSE  Disable PIN
// pPinValue         : Pointer to zero terminated string
//                     Specifies the PIN value of the PIN to be enabled/disabled; 
//                     the protection setting will only be changed 
//                     if this value is successfully verified by the SIM
//
// Return   
// true         : success
// false        : - callback function not registered (see CmdApi_Register())
//                - PinId invalide
//                - string of the pPinValue too long or not zero terminated                
//                
//-----------------------------------------------------------------------------
CONMANDLL_API bool CmdApi_SetPinProtectionReq
(
    unsigned int    PinId,
    bool            ProtectionSetting,
    unsigned char   *pPinValue
);

//-----------------------------------------------------------------------------
// CmdApi_VerifyPinReq
//
// This function verifies the PIN before accessing the SIM contents
// (see Response Message CMDAPI_MSG_MT_VERIFY_PIN_RSP)
//
// Parameters
// PinId        : Specifies the ID of the PIN to be verified
//                (valid values 1 = PIN1, 2 = PIN2)
// pPinValue    : Pointer to zero terminated string
//                Specifies the PIN value of the PIN to be verified
//
// Return   
// true         : success
// false        : - callback function not registered (see CmdApi_Register())
//                - PinId invalide
//                - string of the pPinValue too long or not zero terminated                
//                
//-----------------------------------------------------------------------------
CONMANDLL_API bool CmdApi_VerifyPinReq
(
    unsigned int    PinId,
    unsigned char   *pPinValue
);


#ifdef __cplusplus
}
#endif  /* __cplusplus */
