///////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2011 Cinterion Wireless Modules GmbH
//
// This software is protected by international intellectual property laws and
// treaties.
// Cinterion Wireless Modules GmbH ("Cinterion") grants to customer a limited,
// non-exclusive, non-transferable, royalty-free license to use the source code
// as a sample implementation and for the purpose of evaluation, analysis and
// to provide feedback thereon to Cinterion. Any right, title and interest in
// and to the source code, other than those expressly granted to the customer
// under this agreement, shall remain vested with Cinterion. This license may
// be terminated by Cinterion at any time in writing. Customer undertakes not
// to provide third parties with the source code. In particular, Customer is
// not allowed to sell the source code.
// The information contained in this document is considered the CONFIDENTIAL
// and PROPRIETARY information of Cinterion Wireless Modules GmbH and may not
// be disclosed or discussed with anyone who is not employed by Cinterion
// Wireless Modules GmbH, unless the individual/ company:
//
// i)  has an express need to know such information, and
//
// ii) disclosure of information is subject to the terms of a duly executed
//     Confidentiality and Non-Disclosure Agreement between Cinterion Wireless
//     Modules GmbH and the individual / company.
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
// EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
// CINTERION, ITS LEGAL REPRESENTATIVES AND VICARIOUS AGENTS SHALL 
// IRRESPECTIVE OF THE LEGAL GROUND  ONLY BE LIABLE FOR DAMAGES IF THE DAMAGE
// WAS CAUSED THROUGH CULPABLE BREACH OF A MAJOR CONTRACTUAL OBLIGATION
// (CARDINAL DUTY), I.E. A DUTY THE FULFILMENT OF WHICH ALLOWS THE PROPER
// EXECUTION OF THE RESPECTIVE AGREEMENT IN THE FIRST PLACE OR THE BREACH OF
// WHICH PUTS THE ACHIEVEMENT OF THE PURPOSE OF THE AGREEMENT AT STAKE,
// RESPECTIVELY, AND ON THE FULFILMENT OF WHICH THE RECIPIENT THEREFORE MAY
// RELY ON OR WAS CAUSED BY GROSS NEGLIGENCE OR INTENTIONALLY. ANY FURTHER
// LIABILITY FOR DAMAGES SHALL  IRRESPECTIVE OF THE LEGAL GROUND  BE
// EXCLUDED. IN THE EVENT THAT CINTERION IS LIABLE FOR THE VIOLATION OF A MAJOR
// CONTRACTUAL OBLIGATION IN THE ABSENCE OF GROSS NEGLIGENCE OR WILFUL CONDUCT,
// SUCH LIABILITY FOR DAMAGE SHALL BE LIMITED TO AN EXTENT WHICH, AT THE TIME
// WHEN THE RESPECTIVE AGREEMENT IS CONCLUDED, CINTERION SHOULD NORMALLY EXPECT
// TO ARISE DUE TO CIRCUMSTANCES THAT THE PARTIES HAD KNOWLEDGE OF AT SUCH
// POINT IN TIME. CINTERION SHALL IN NO EVENT BE LIABLE FOR INDIRECT AND
// CONSEQUENTIAL DAMAGES OR LOSS OF PROFIT. CINTERION SHALL IN NO EVENT BE
// LIABLE FOR AN AMOUNT EXCEEDING  20,000.00 PER EVENT OF DAMAGE. WITHIN THE
// BUSINESS RELATIONSHIP THE OVERALL LIABILITY SHALL BE LIMITED TO A TOTAL OF
//  100,000.00. CLAIMS FOR DAMAGES SHALL BECOME TIME-BARRED AFTER ONE YEAR AS
// OF THE BEGINNING OF THE STATUTORY LIMITATION PERIOD. IRRESPECTIVE OF THE
// LICENSEES KNOWLEDGE OR GROSS NEGLIGENT LACK OF KNOWLEDGE OF THE
// CIRCUMSTANCES GIVING RISE FOR A LIABILITY ANY CLAIMS SHALL BECOME
// TIME-BARRED AFTER FIVE YEARS AS OF THE LIABILITY AROSE. THE AFOREMENTIONED
// LIMITATION OR EXCLUSION OF LIABILITY SHALL NOT APPLY IN THE CASE OF CULPABLE
// INJURY TO LIFE, BODY OR HEALTH, IN CASE OF INTENTIONAL ACTS, UNDER THE
// LIABILITY PROVISIONS OF THE GERMAN PRODUCT LIABILITY ACT
// (PRODUKTHAFTUNGSGESETZ) OR IN CASE OF A CONTRACTUALLY AGREED OBLIGATION TO
// ASSUME LIABILITY IRRESPECTIVE OF ANY FAULT (GUARANTEE).
//
// IN THE EVENT OF A CONFLICT BETWEEN THE PROVISIONS OF THIS AGREEMENT AND
// ANOTHER AGREEMENT REGARDING THE SOURCE CODE (EXCEPT THE GENERAL TERMS AND
// CONDITIONS OF CINTERION) THE OTHER AGREEMENT SHALL PREVAIL.
//
// All rights created by patent grant or registration of a utility model or
// design patent are reserved.
//
///////////////////////////////////////////////////////////////////////////////
#include "CmdApi.h"

static CCmdApi* pCCmdApi = NULL;

CCmdApi::CCmdApi(void)
{
    pCCmdApi = this;
    CmdApi_Register (CmdApiProcessMsg);
}

CCmdApi::~CCmdApi(void)
{
}

CCmdApi& CCmdApi::GetInstance()
{
    return *pCCmdApi;
}

CMessage *CCmdApi::ProcessMsg_CMDAPI_MSGCLASS_NT_DEVICE
(
    PCMD_API_MSG_T  pCmdApiMsg
)
{
    CMsgDevice  *pMsg = new CMsgDevice(
                                pCmdApiMsg->msg_class,
                                pCmdApiMsg->msg_type,
                                pCmdApiMsg->msg.msg_dt_device.pList,
                                pCmdApiMsg->msg.msg_dt_device.pReason
                            );

    return pMsg;
}

CMessage *CCmdApi::ProcessMsg_CMDAPI_MSGCLASS_NT_MESSAGE
(
    PCMD_API_MSG_T  pCmdApiMsg
)
{
    CMessage *pMsg = NULL;
    switch (pCmdApiMsg->msg_type) {
        case CMDAPI_MSG_MT_START_CONNECTION_RSP:
            pMsg = new CMsgStartConnectionRsp(
                           pCmdApiMsg->msg_class,
                           pCmdApiMsg->msg_type,
                           pCmdApiMsg->msg.msg_mt_start_connection_rsp.DataHandle,
                           pCmdApiMsg->msg.msg_mt_start_connection_rsp.Result,
                           pCmdApiMsg->msg.msg_mt_start_connection_rsp.Error
                       ); 
            break;
        case CMDAPI_MSG_MT_ABORT_CONNECTION_RSP:
            pMsg = new CMsgAbortConnectionRsp(
                           pCmdApiMsg->msg_class,
                           pCmdApiMsg->msg_type,
                           pCmdApiMsg->msg.msg_mt_abort_connection_rsp.Result,
                           pCmdApiMsg->msg.msg_mt_abort_connection_rsp.Error
                       );
            break;
        case CMDAPI_MSG_MT_STOP_CONNECTION_RSP:
            pMsg = new CMsgStopConnectionRsp (
                           pCmdApiMsg->msg_class,
                           pCmdApiMsg->msg_type,
                           pCmdApiMsg->msg.msg_mt_stop_connection_rsp.Result,
                           pCmdApiMsg->msg.msg_mt_stop_connection_rsp.Error
                       );
            break;

        case CMDAPI_MSG_MT_STATUS_CONNECTION_IND:
            pMsg = new CMsgStatusConnectionInd (
                           pCmdApiMsg->msg_class,
                           pCmdApiMsg->msg_type,
                           pCmdApiMsg->msg.msg_mt_status_connection_ind.Connection_Status
                       );
            break;

        case CMDAPI_MSG_MT_SIGNAL_STRENGTH_RSP:
            pMsg = new CGetSignalStrengthRsp (
                           pCmdApiMsg->msg_class,
                           pCmdApiMsg->msg_type,
                           pCmdApiMsg->msg.msg_mt_signal_strength_rsp.SignalStrength,
                           pCmdApiMsg->msg.msg_mt_signal_strength_rsp.RadioIf,
                           pCmdApiMsg->msg.msg_mt_signal_strength_rsp.Result,
                           pCmdApiMsg->msg.msg_mt_signal_strength_rsp.Error
                       );
            break;
        case CMDAPI_MSG_MT_SET_EVENT_REPORT_RSP:
            pMsg = new CSetEventReportRsp (
                           pCmdApiMsg->msg_class,
                           pCmdApiMsg->msg_type,
                           pCmdApiMsg->msg.msg_mt_set_event_report_rsp.Result,
                           pCmdApiMsg->msg.msg_mt_set_event_report_rsp.Error
                       );
            break;
        case CMDAPI_MSG_MT_EVENT_REPORT_IND:
            pMsg = new CEventReportInd(
                           pCmdApiMsg->msg_class,
                           pCmdApiMsg->msg_type,
                           pCmdApiMsg->msg.msg_mt_event_report_ind.SignalStrength,
                           pCmdApiMsg->msg.msg_mt_event_report_ind.RadioIf
                       );
            break;
        case CMDAPI_MSG_MT_GET_PIN_STATUS_RSP:
            pMsg = new CGetPinStatusRsp(
                           pCmdApiMsg->msg_class,
                           pCmdApiMsg->msg_type,
                           pCmdApiMsg->msg.msg_mt_get_pin_status_rsp.Result,
                           pCmdApiMsg->msg.msg_mt_get_pin_status_rsp.Error,
                           pCmdApiMsg->msg.msg_mt_get_pin_status_rsp.IsPin1Status,
                           pCmdApiMsg->msg.msg_mt_get_pin_status_rsp.IsPin2Status,
                           pCmdApiMsg->msg.msg_mt_get_pin_status_rsp.Pin1Status,
                           pCmdApiMsg->msg.msg_mt_get_pin_status_rsp.Pin1VerifyRetriesLeft,
                           pCmdApiMsg->msg.msg_mt_get_pin_status_rsp.Pin1UnblockRetriesLeft,
                           pCmdApiMsg->msg.msg_mt_get_pin_status_rsp.Pin2Status,
                           pCmdApiMsg->msg.msg_mt_get_pin_status_rsp.Pin2VerifyRetriesLeft,
                           pCmdApiMsg->msg.msg_mt_get_pin_status_rsp.Pin2UnblockRetriesLeft
                       );
            break;
        case CMDAPI_MSG_MT_SET_PIN_PROTECTION_RSP:
            pMsg = new CSetPinProtectionRsp(
                           pCmdApiMsg->msg_class,
                           pCmdApiMsg->msg_type,
                           pCmdApiMsg->msg.msg_mt_set_pin_protection_rsp.Result,
                           pCmdApiMsg->msg.msg_mt_set_pin_protection_rsp.Error,
                           pCmdApiMsg->msg.msg_mt_set_pin_protection_rsp.IsRetriesLeft,
                           pCmdApiMsg->msg.msg_mt_set_pin_protection_rsp.VerifyRetriesLeft,
                           pCmdApiMsg->msg.msg_mt_set_pin_protection_rsp.UnblockRetriesLeft
                       );
            break;
        case CMDAPI_MSG_MT_VERIFY_PIN_RSP:
            pMsg = new CVerifyPinRsp(
                           pCmdApiMsg->msg_class,
                           pCmdApiMsg->msg_type,
                           pCmdApiMsg->msg.msg_mt_verify_pin_rsp.Result,
                           pCmdApiMsg->msg.msg_mt_verify_pin_rsp.Error,
                           pCmdApiMsg->msg.msg_mt_verify_pin_rsp.IsRetriesLeft,
                           pCmdApiMsg->msg.msg_mt_verify_pin_rsp.VerifyRetriesLeft,
                           pCmdApiMsg->msg.msg_mt_verify_pin_rsp.UnblockRetriesLeft
                       );
            break;

        case CMDAPI_MSG_MT_SERVING_SYSTEM_RSP:
            pMsg = new CServingSystemRsp(
                           pCmdApiMsg->msg_class,
                           pCmdApiMsg->msg_type,
                           pCmdApiMsg->msg.msg_mt_serving_system_rsp.Result,
                           pCmdApiMsg->msg.msg_mt_serving_system_rsp.Error,
                           pCmdApiMsg->msg.msg_mt_serving_system_rsp.RegistrationState,
                           pCmdApiMsg->msg.msg_mt_serving_system_rsp.CSAttachState,
                           pCmdApiMsg->msg.msg_mt_serving_system_rsp.PSAttachState,
                           pCmdApiMsg->msg.msg_mt_serving_system_rsp.RegisteredNetwork,
                           pCmdApiMsg->msg.msg_mt_serving_system_rsp.NumRadioIfs,
                           pCmdApiMsg->msg.msg_mt_serving_system_rsp.pRadioIfslist
                       );
            break;

        case CMDAPI_MSG_MT_SERVING_SYSTEM_IND:
            pMsg = new CServingSystemInd(
                           pCmdApiMsg->msg_class,
                           pCmdApiMsg->msg_type,
                           pCmdApiMsg->msg.msg_mt_serving_system_ind.RegistrationState,
                           pCmdApiMsg->msg.msg_mt_serving_system_ind.CSAttachState,
                           pCmdApiMsg->msg.msg_mt_serving_system_ind.PSAttachState,
                           pCmdApiMsg->msg.msg_mt_serving_system_ind.RegisteredNetwork,
                           pCmdApiMsg->msg.msg_mt_serving_system_ind.NumRadioIfs,
                           pCmdApiMsg->msg.msg_mt_serving_system_ind.pRadioIfslist
                       );
            break;

    }

    return pMsg;
}

void CCmdApi::ProcessMsg(
    PCMD_API_MSG_T  pCmdApiMsg
)
{
    CMessage    *pMsg;

    switch (pCmdApiMsg->msg_class) {
        case CMDAPI_MSGCLASS_NT_DEVICE:
            pMsg = ProcessMsg_CMDAPI_MSGCLASS_NT_DEVICE (pCmdApiMsg);
            break;

        case CMDAPI_MSGCLASS_NT_MESSAGE:
            pMsg = ProcessMsg_CMDAPI_MSGCLASS_NT_MESSAGE (pCmdApiMsg);
            break;
    }

    if (pMsg != NULL) {
        NotifyMsg(pMsg);
    }
    return;
}


// --------------------------------------------------------------------------
// callback function
// CmdApiProcessMsg() was registred by call of CmdApi_Register()
// This function reveives messages from the ConManDll interface.
// --------------------------------------------------------------------------
void CmdApiProcessMsg(
    PCMD_API_MSG_T  pCmdApiMsg
)
{
    CCmdApi::GetInstance().ProcessMsg(pCmdApiMsg);
    return;
}