///////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2011 Cinterion Wireless Modules GmbH
//
// This software is protected by international intellectual property laws and
// treaties.
// Cinterion Wireless Modules GmbH ("Cinterion") grants to customer a limited,
// non-exclusive, non-transferable, royalty-free license to use the source code
// as a sample implementation and for the purpose of evaluation, analysis and
// to provide feedback thereon to Cinterion. Any right, title and interest in
// and to the source code, other than those expressly granted to the customer
// under this agreement, shall remain vested with Cinterion. This license may
// be terminated by Cinterion at any time in writing. Customer undertakes not
// to provide third parties with the source code. In particular, Customer is
// not allowed to sell the source code.
// The information contained in this document is considered the CONFIDENTIAL
// and PROPRIETARY information of Cinterion Wireless Modules GmbH and may not
// be disclosed or discussed with anyone who is not employed by Cinterion
// Wireless Modules GmbH, unless the individual/ company:
//
// i)  has an express need to know such information, and
//
// ii) disclosure of information is subject to the terms of a duly executed
//     Confidentiality and Non-Disclosure Agreement between Cinterion Wireless
//     Modules GmbH and the individual / company.
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
// EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
// CINTERION, ITS LEGAL REPRESENTATIVES AND VICARIOUS AGENTS SHALL 
// IRRESPECTIVE OF THE LEGAL GROUND  ONLY BE LIABLE FOR DAMAGES IF THE DAMAGE
// WAS CAUSED THROUGH CULPABLE BREACH OF A MAJOR CONTRACTUAL OBLIGATION
// (CARDINAL DUTY), I.E. A DUTY THE FULFILMENT OF WHICH ALLOWS THE PROPER
// EXECUTION OF THE RESPECTIVE AGREEMENT IN THE FIRST PLACE OR THE BREACH OF
// WHICH PUTS THE ACHIEVEMENT OF THE PURPOSE OF THE AGREEMENT AT STAKE,
// RESPECTIVELY, AND ON THE FULFILMENT OF WHICH THE RECIPIENT THEREFORE MAY
// RELY ON OR WAS CAUSED BY GROSS NEGLIGENCE OR INTENTIONALLY. ANY FURTHER
// LIABILITY FOR DAMAGES SHALL  IRRESPECTIVE OF THE LEGAL GROUND  BE
// EXCLUDED. IN THE EVENT THAT CINTERION IS LIABLE FOR THE VIOLATION OF A MAJOR
// CONTRACTUAL OBLIGATION IN THE ABSENCE OF GROSS NEGLIGENCE OR WILFUL CONDUCT,
// SUCH LIABILITY FOR DAMAGE SHALL BE LIMITED TO AN EXTENT WHICH, AT THE TIME
// WHEN THE RESPECTIVE AGREEMENT IS CONCLUDED, CINTERION SHOULD NORMALLY EXPECT
// TO ARISE DUE TO CIRCUMSTANCES THAT THE PARTIES HAD KNOWLEDGE OF AT SUCH
// POINT IN TIME. CINTERION SHALL IN NO EVENT BE LIABLE FOR INDIRECT AND
// CONSEQUENTIAL DAMAGES OR LOSS OF PROFIT. CINTERION SHALL IN NO EVENT BE
// LIABLE FOR AN AMOUNT EXCEEDING  20,000.00 PER EVENT OF DAMAGE. WITHIN THE
// BUSINESS RELATIONSHIP THE OVERALL LIABILITY SHALL BE LIMITED TO A TOTAL OF
//  100,000.00. CLAIMS FOR DAMAGES SHALL BECOME TIME-BARRED AFTER ONE YEAR AS
// OF THE BEGINNING OF THE STATUTORY LIMITATION PERIOD. IRRESPECTIVE OF THE
// LICENSEES KNOWLEDGE OR GROSS NEGLIGENT LACK OF KNOWLEDGE OF THE
// CIRCUMSTANCES GIVING RISE FOR A LIABILITY ANY CLAIMS SHALL BECOME
// TIME-BARRED AFTER FIVE YEARS AS OF THE LIABILITY AROSE. THE AFOREMENTIONED
// LIMITATION OR EXCLUSION OF LIABILITY SHALL NOT APPLY IN THE CASE OF CULPABLE
// INJURY TO LIFE, BODY OR HEALTH, IN CASE OF INTENTIONAL ACTS, UNDER THE
// LIABILITY PROVISIONS OF THE GERMAN PRODUCT LIABILITY ACT
// (PRODUKTHAFTUNGSGESETZ) OR IN CASE OF A CONTRACTUALLY AGREED OBLIGATION TO
// ASSUME LIABILITY IRRESPECTIVE OF ANY FAULT (GUARANTEE).
//
// IN THE EVENT OF A CONFLICT BETWEEN THE PROVISIONS OF THIS AGREEMENT AND
// ANOTHER AGREEMENT REGARDING THE SOURCE CODE (EXCEPT THE GENERAL TERMS AND
// CONDITIONS OF CINTERION) THE OTHER AGREEMENT SHALL PREVAIL.
//
// All rights created by patent grant or registration of a utility model or
// design patent are reserved.
//
///////////////////////////////////////////////////////////////////////////////
#pragma once
#include <sstream>
#include <string>
#include <vector>
#include "stdafx.h"
#include "ConManDll.h"


// ----------------------------------------------------------------------------
// Error Strings
// ----------------------------------------------------------------------------
static const std::string CMDAPI_ERROR_STRINGS[] = 
{
    "None",
    "Malformed Message",
    "No Memmory",
    "Internal",
    "Aborted",
    "Client IDs Exausted",
    "Unabortable Transaction",
    "Invalid Client ID",
    "No Thresholds",
    "Invalid Handle",
    "Invalid Profile",
    "Invalid Pin ID",
    "Incorrect Pin",
    "No Network Found",
    "Call Failed",
    "Out Of Call",
    "Not Provisioned",
    "Missing Arguement",
    "UNSUPPORTED ERROR CODE",
    "Arguement Too Long",
    "UNSUPPORTED ERROR CODE",
    "UNSUPPORTED ERROR CODE",
    "Invalid Transaction ID",
    "Device In Use",
    "OP Network Unsupported",
    "OP Device Unsupported",
    "No Effect",
    "No Free Profile",
    "Invalid PDP Type",
    "Invalid Technology Preference",
    "Invalid Profile Type",
    "Invalid Service Type",
    "Invalid Register Action",
    "Invalid PS Attach Action",
    "Authentication Failed",
    "Pin Blocked",
    "Pin Permanently Blocked",
    "Sim Not Initialized",
    "Max QOS Requests In Use",
    "Incorrect Flow Filter",
    "Network QOS Unaware",
    "Invalid QOS Id",
    "Num QOS Ids",
    "QOS Unavailable",
    "Flow Suspended",
    "Invalid Data Format"
};


// ----------------------------------------------------------------------------
// Radio Interface Technology Strings
// ----------------------------------------------------------------------------
static const char *CMDAPI_RADIO_IF_TECH_STRINGS[] = 
{
    "No Service",
    "CDMA2000 1x",
    "CDMA2000 HRPD",
    "AMPS",
    "GSM",
    "UMTS"
};


// ----------------------------------------------------------------------------
// Network Registration States Strings
// ----------------------------------------------------------------------------
static const char *CMDAPI_REG_STATE_STRINGS[] = 
{
    "Not Registered",
    "Registered",
    "Not Registered, searching",
    "Registration Denied",
    "Registration unknown"
};


// ----------------------------------------------------------------------------
// Pin Status
// ----------------------------------------------------------------------------
static const char *CMDAPI_PIN_STATUS_STRINGS[] = 
{
    "Pin Not Initialized",
    "Pin Enabled, Not Verified",
    "Pin Enabled, Verified",
    "Pin Disabled",
    "Pin Blocked",
    "Pin Permanently Blocked",
    "Pin Unblocked",
    "Pin Changed"
};


class CMessage
{
public:
    CMessage(DWORD dwClass, DWORD dwType) {m_Class = dwClass; m_Type = dwType;};  
    virtual ~CMessage() {};
    DWORD GetType() {return m_Type;};
    DWORD GetClass() {return m_Class;};

private:
    DWORD   m_Class;
    DWORD   m_Type;   

};

class CRspMessage : public CMessage
{
public:
    CRspMessage(DWORD dwClass, DWORD dwType, DWORD dwResult, DWORD dwError)
        : CMessage(dwClass, dwType), m_dwResult(dwResult), m_dwError(dwError)
    {
    };

    DWORD GetResult()     {return m_dwResult;};
    DWORD GetError()      {return m_dwError;};

private:
    DWORD   m_dwResult;
    DWORD   m_dwError;
};


class CMsgDevice : public CMessage
{
public:
    CMsgDevice(DWORD dwClass, DWORD dwType, unsigned char *pAdapterList, unsigned char *pReason)
        : CMessage(dwClass, dwType)
        {
            while (*pAdapterList != '\0') {
                m_networkAdapters.push_back((char *)pAdapterList);
                pAdapterList += strlen((char *)pAdapterList) + 1;
            }
            m_Reason = (char *)pReason;
        };
    ~CMsgDevice()
        {
            m_networkAdapters.clear();
        }
    std::vector<std::string> GetNetworkAdapters () {return m_networkAdapters;};
    std::string GetReason() {return m_Reason;};
private:
    std::vector<std::string>    m_networkAdapters;
    std::string                 m_Reason;

};

class CMsgStartConnectionRsp : public CRspMessage
{
public:
    CMsgStartConnectionRsp(DWORD dwClass, DWORD dwType, DWORD dwDataHandle, DWORD dwResult, DWORD dwError)
        : CRspMessage(dwClass, dwType, dwResult, dwError)
        {
            m_dwDataHandle  = dwDataHandle;
        };
    ~CMsgStartConnectionRsp(){};

    DWORD GetDataHandle() {return m_dwDataHandle;};

private:
    DWORD   m_dwDataHandle;
};

class CMsgAbortConnectionRsp : public CRspMessage
{
public:
    CMsgAbortConnectionRsp(DWORD dwClass, DWORD dwType, DWORD dwResult, DWORD dwError)
        : CRspMessage(dwClass, dwType, dwResult, dwError)
        {
        };
    ~CMsgAbortConnectionRsp(){};

private:
};

class CMsgStopConnectionRsp : public CRspMessage
{
public:
    CMsgStopConnectionRsp(DWORD dwClass, DWORD dwType, DWORD dwResult, DWORD dwError)
        : CRspMessage(dwClass, dwType, dwResult, dwError)
        {
        };
    ~CMsgStopConnectionRsp(){};

private:
};

class CMsgStatusConnectionInd : public CMessage
{
public:
    CMsgStatusConnectionInd(DWORD dwClass, DWORD dwType, unsigned int Connection_Status)
        : CMessage(dwClass, dwType)
        {
            m_ConnectionStatus = Connection_Status;
        };
    ~CMsgStatusConnectionInd(){};
    unsigned int GetConnectionStatus () {return m_ConnectionStatus;};
private:
    unsigned int    m_ConnectionStatus;
};

class CGetSignalStrengthRsp : public CRspMessage
{
public:
    CGetSignalStrengthRsp(DWORD dwClass, DWORD dwType, signed char SignalStrength, unsigned char RadioIf, DWORD dwResult, DWORD dwError)
        : CRspMessage(dwClass, dwType, dwResult, dwError)
        {
            m_SignalStrength    = SignalStrength;
            m_RadioIf           = RadioIf;
        };
    ~CGetSignalStrengthRsp(){};

    signed char   GetSignalStrength()   {return m_SignalStrength;};
    unsigned char GetRadioIf()          {return m_RadioIf;};

private:
    signed char     m_SignalStrength;
    unsigned char   m_RadioIf;
};


class CSetEventReportRsp : public CRspMessage
{
public:
    CSetEventReportRsp(DWORD dwClass, DWORD dwType, DWORD dwResult, DWORD dwError)
        : CRspMessage(dwClass, dwType, dwResult, dwError)
        {
        };
    ~CSetEventReportRsp(){};

private:
};

class CEventReportInd : public CMessage
{
public:
    CEventReportInd(DWORD dwClass, DWORD dwType, signed char SignalStrength, unsigned char RadioIf)
        : CMessage(dwClass, dwType)
        {
            m_SignalStrength    = SignalStrength;
            m_RadioIf           = RadioIf;
        };
    ~CEventReportInd(){};

    signed char   GetSignalStrength()   {return m_SignalStrength;};
    unsigned char GetRadioIf()          {return m_RadioIf;};

private:
    signed char     m_SignalStrength;
    unsigned char   m_RadioIf;
};

class CServingSystemRsp : public CRspMessage
{
public:
    CServingSystemRsp(DWORD dwClass, DWORD dwType, DWORD dwResult, DWORD dwError,
                        unsigned char RegistrationState,
                        unsigned char CSAttachState,
                        unsigned char PSAttachState,
                        unsigned char RegisteredNetwork,
                        unsigned char NumRadioIfs,
                        unsigned char *pRadioIfslist
                     )
        : CRspMessage(dwClass, dwType, dwResult, dwError)
        {
            m_RegistrationState = RegistrationState;
            m_CSAttachState = CSAttachState;
            m_PSAttachState = PSAttachState;
            m_RegisteredNetwork = RegisteredNetwork;
            m_NumRadioIfs = NumRadioIfs;

            if (m_NumRadioIfs > 0) {
                for (int i = 0; i < m_NumRadioIfs; i++) {
                    m_RadioIfslist.push_back(pRadioIfslist[i]);
                }
            }
        };
    ~CServingSystemRsp(){};

    unsigned char GetRegistrationState()         {return m_RegistrationState;};
    unsigned char GetCSAttachState()             {return m_CSAttachState;};
    unsigned char GetPSAttachState()             {return m_PSAttachState;};
    unsigned char GetRegisteredNetwork()         {return m_RegisteredNetwork;};
    std::vector<unsigned char> GetRadioIfslist() {return m_RadioIfslist;};

private:
    unsigned char       m_RegistrationState;
    unsigned char       m_CSAttachState;
    unsigned char       m_PSAttachState;
    unsigned char       m_RegisteredNetwork;
    unsigned char       m_NumRadioIfs;
    std::vector<unsigned char>    m_RadioIfslist;
};

class CServingSystemInd : public CMessage
{
public:
    CServingSystemInd(DWORD dwClass, DWORD dwType, 
                        unsigned char RegistrationState,
                        unsigned char CSAttachState,
                        unsigned char PSAttachState,
                        unsigned char RegisteredNetwork,
                        unsigned char NumRadioIfs,
                        unsigned char *pRadioIfslist
                     )
        : CMessage(dwClass, dwType)
        {
            m_RegistrationState = RegistrationState;
            m_CSAttachState = CSAttachState;
            m_PSAttachState = PSAttachState;
            m_RegisteredNetwork = RegisteredNetwork;
            m_NumRadioIfs = NumRadioIfs;

            if (m_NumRadioIfs > 0) {
                for (int i = 0; i < m_NumRadioIfs; i++) {
                    m_RadioIfslist.push_back(pRadioIfslist[i]);
                }
            }
        };
    ~CServingSystemInd(){};

    unsigned char GetRegistrationState()         {return m_RegistrationState;};
    unsigned char GetCSAttachState()             {return m_CSAttachState;};
    unsigned char GetPSAttachState()             {return m_PSAttachState;};
    unsigned char GetRegisteredNetwork()         {return m_RegisteredNetwork;};
    std::vector<unsigned char> GetRadioIfslist() {return m_RadioIfslist;};

private:
    unsigned char       m_RegistrationState;
    unsigned char       m_CSAttachState;
    unsigned char       m_PSAttachState;
    unsigned char       m_RegisteredNetwork;
    unsigned char       m_NumRadioIfs;
    std::vector<unsigned char>    m_RadioIfslist;
};

class CGetPinStatusRsp : public CRspMessage
{
public:
    CGetPinStatusRsp(DWORD dwClass, DWORD dwType, DWORD dwResult, DWORD dwError,
                        bool            IsPin1Status,
                        bool            IsPin2Status,
                        unsigned char   Pin1Status,
                        unsigned char   Pin1VerifyRetriesLeft,
                        unsigned char   Pin1UnblockRetriesLeft,
                        unsigned char   Pin2Status,
                        unsigned char   Pin2VerifyRetriesLeft,
                        unsigned char   Pin2UnblockRetriesLeft
                    )
        : CRspMessage(dwClass, dwType, dwResult, dwError)
        {
            m_IsPin1Status              = IsPin1Status;
            m_IsPin2Status              = IsPin2Status;

            m_Pin1Status                = Pin1Status;
            m_Pin1VerifyRetriesLeft     = Pin1VerifyRetriesLeft;
            m_Pin1UnblockRetriesLeft    = Pin1UnblockRetriesLeft;

            m_Pin2Status                = Pin2Status;
            m_Pin2VerifyRetriesLeft     = Pin2VerifyRetriesLeft;
            m_Pin2UnblockRetriesLeft    = Pin2UnblockRetriesLeft;

        };
    ~CGetPinStatusRsp() {};

    bool IsPin1Status() { return m_IsPin1Status; };
    bool IsPin2Status() { return m_IsPin2Status; };

    unsigned char GetPin1Status() { return m_Pin1Status; };
    unsigned char GetPin1VerifyRetriesLeft() { return m_Pin1VerifyRetriesLeft; };
    unsigned char GetPin1UnblockRetriesLeft() { return m_Pin1UnblockRetriesLeft; };

    unsigned char GetPin2Status() { return m_Pin2Status; };
    unsigned char GetPin2VerifyRetriesLeft() { return m_Pin2VerifyRetriesLeft; };
    unsigned char GetPin2UnblockRetriesLeft() { return m_Pin2UnblockRetriesLeft; };

private:
    bool                m_IsPin1Status;
    bool                m_IsPin2Status;

    unsigned char       m_Pin1Status;
    unsigned char       m_Pin1VerifyRetriesLeft;
    unsigned char       m_Pin1UnblockRetriesLeft;

    unsigned char       m_Pin2Status;
    unsigned char       m_Pin2VerifyRetriesLeft;
    unsigned char       m_Pin2UnblockRetriesLeft;
};

class CChangePinRsp : public CRspMessage
{
public:
    CChangePinRsp(DWORD dwClass, DWORD dwType, DWORD dwResult, DWORD dwError,
                     bool IsRetriesLeft,
                     unsigned char VerifyRetriesLeft,
                     unsigned char UnblockRetriesLeft
                 )
        : CRspMessage(dwClass, dwType, dwResult, dwError)
        {
            m_IsRetriesLeft         = IsRetriesLeft;
            m_VerifyRetriesLeft     = VerifyRetriesLeft;
            m_UnblockRetriesLeft    = UnblockRetriesLeft;
        };

    ~CChangePinRsp(){};

    DWORD GetResult()     {return m_dwResult;};
    DWORD GetError()      {return m_dwError;};

    bool IsRetriesLeft() { return m_IsRetriesLeft; };
    unsigned char GetVerifyRetriesLeft() { return m_VerifyRetriesLeft; };
    unsigned char GetUnblockRetriesLeft() { return m_UnblockRetriesLeft; };

private:    
    DWORD   m_dwResult;
    DWORD   m_dwError;

    bool                m_IsRetriesLeft;
    unsigned char       m_VerifyRetriesLeft;
    unsigned char       m_UnblockRetriesLeft;
};


class CSetPinProtectionRsp : public CRspMessage
{
public:
    CSetPinProtectionRsp(DWORD dwClass, DWORD dwType, DWORD dwResult, DWORD dwError,
                     bool IsRetriesLeft,
                     unsigned char VerifyRetriesLeft,
                     unsigned char UnblockRetriesLeft
                 )
        : CRspMessage(dwClass, dwType, dwResult, dwError)
        {
            m_IsRetriesLeft         = IsRetriesLeft;
            m_VerifyRetriesLeft     = VerifyRetriesLeft;
            m_UnblockRetriesLeft    = UnblockRetriesLeft;
        };

    ~CSetPinProtectionRsp(){};

    bool IsRetriesLeft() { return m_IsRetriesLeft; };
    unsigned char GetVerifyRetriesLeft() { return m_VerifyRetriesLeft; };
    unsigned char GetUnblockRetriesLeft() { return m_UnblockRetriesLeft; };

private:    
    bool                m_IsRetriesLeft;
    unsigned char       m_VerifyRetriesLeft;
    unsigned char       m_UnblockRetriesLeft;
};

class CUnblockPinRsp : public CRspMessage
{
public:
    CUnblockPinRsp(DWORD dwClass, DWORD dwType, DWORD dwResult, DWORD dwError,
                     bool IsRetriesLeft,
                     unsigned char VerifyRetriesLeft,
                     unsigned char UnblockRetriesLeft
                 )
        : CRspMessage(dwClass, dwType, dwResult, dwError)
        {
            m_IsRetriesLeft         = IsRetriesLeft;
            m_VerifyRetriesLeft     = VerifyRetriesLeft;
            m_UnblockRetriesLeft    = UnblockRetriesLeft;
        };

    ~CUnblockPinRsp(){};

    bool IsRetriesLeft() { return m_IsRetriesLeft; };
    unsigned char GetVerifyRetriesLeft() { return m_VerifyRetriesLeft; };
    unsigned char GetUnblockRetriesLeft() { return m_UnblockRetriesLeft; };

private:    
    bool                m_IsRetriesLeft;
    unsigned char       m_VerifyRetriesLeft;
    unsigned char       m_UnblockRetriesLeft;
};

class CVerifyPinRsp : public CRspMessage
{
public:
    CVerifyPinRsp(DWORD dwClass, DWORD dwType, DWORD dwResult, DWORD dwError,
                     bool IsRetriesLeft,
                     unsigned char VerifyRetriesLeft,
                     unsigned char UnblockRetriesLeft
                 )
        : CRspMessage(dwClass, dwType, dwResult, dwError)
        {
            m_IsRetriesLeft         = IsRetriesLeft;
            m_VerifyRetriesLeft     = VerifyRetriesLeft;
            m_UnblockRetriesLeft    = UnblockRetriesLeft;
        };

    ~CVerifyPinRsp(){};

    bool IsRetriesLeft() { return m_IsRetriesLeft; };
    unsigned char GetVerifyRetriesLeft() { return m_VerifyRetriesLeft; };
    unsigned char GetUnblockRetriesLeft() { return m_UnblockRetriesLeft; };

private:    
    bool                m_IsRetriesLeft;
    unsigned char       m_VerifyRetriesLeft;
    unsigned char       m_UnblockRetriesLeft;
};









class CReqStartConnection
{
public:
    CReqStartConnection () : m_AuthPref(AUTHPREF_AUTO) {};

    void SetPrimaryDnsPref(CString PrimaryDnsPref) {m_PrimaryDnsPref = PrimaryDnsPref;};
    void SetSecondaryDnsPref(CString SecondaryDnsPref) {m_SecondaryDnsPref = SecondaryDnsPref;};
    void SetPrimaryNbnsPref(CString PrimaryNbnsPref) {m_PrimaryNbnsPref = PrimaryNbnsPref;};
    void SetSecondaryNbnsPref(CString SecondaryNbnsPref) {m_SecondaryNbnsPref = SecondaryNbnsPref;};
    void SetApnName(CString ApnName) {m_ApnName = ApnName;};
    void SetIpv4AddrPref(CString Ipv4AddrPref) {m_Ipv4AddrPref = Ipv4AddrPref;};
    void SetAuthPref (int AuthPref) {m_AuthPref = AuthPref;};
    void SetAuthPref_PAP () {m_AuthPref = AUTHPREF_PAP;};
    void SetAuthPref_CHAP () {m_AuthPref = AUTHPREF_CHAP;};
    void SetAuthPref_PAP_CHAP () {m_AuthPref = AUTHPREF_PAP_CHAP;};
    void SetUsername(CString Username) {m_Username = Username;};
    void SetPassword(CString Password) {m_Password = Password;};

    unsigned int SendRequest (void) 
        {
            return CmdApi_StartConnectionReq (
                        m_PrimaryDnsPref.empty() ? NULL : (unsigned char *)m_PrimaryDnsPref.c_str(),
                        m_SecondaryDnsPref.empty() ? NULL : (unsigned char *)m_SecondaryDnsPref.c_str(),
                        m_PrimaryNbnsPref.empty() ? NULL : (unsigned char *)m_PrimaryNbnsPref.c_str(),
                        m_SecondaryNbnsPref.empty() ? NULL : (unsigned char *)m_SecondaryNbnsPref.c_str(),
                        m_ApnName.empty() ? NULL : (unsigned char *)m_ApnName.c_str(),
                        m_Ipv4AddrPref.empty() ? NULL : (unsigned char *)m_Ipv4AddrPref.c_str(),
                        m_AuthPref,
                        m_Username.empty() ? NULL : (unsigned char *)m_Username.c_str(),
                        m_Password.empty() ? NULL : (unsigned char *)m_Password.c_str()
                    );
        };
private:
    std::string m_PrimaryDnsPref;
    std::string m_SecondaryDnsPref;
    std::string m_PrimaryNbnsPref;
    std::string m_SecondaryNbnsPref;
    std::string m_ApnName;
    std::string m_Ipv4AddrPref;
    int         m_AuthPref;
    std::string m_Username;
    std::string m_Password;
};

class CReqAbortConnection
{
public:
    CReqAbortConnection (DWORD dwTxId) : m_dwTxId(dwTxId) {};

    bool SendRequest (void) 
        {
            return CmdApi_AbortConnectionReq (m_dwTxId);
        }
private:
    DWORD   m_dwTxId;
};

class CReqStopConnection
{
public:
    CReqStopConnection (DWORD dwDataHandle) : m_dwDataHandle(dwDataHandle) {};

    bool SendRequest (void) 
        {
            return CmdApi_StopConnectionReq (m_dwDataHandle);
        }
private:
    DWORD   m_dwDataHandle;
};

class CReqGetSignalStrength
{
public:
    CReqGetSignalStrength () {};

    bool SendRequest (void) 
        {
            return CmdApi_GetSignalStrengthReq ();
        }
};

class CReqSetEventReport
{
public:
    CReqSetEventReport () : m_fReport(FALSE), m_NumRangeLimits(0), m_pRangeLimits(NULL) {};
    ~CReqSetEventReport () 
        {
            if (m_pRangeLimits) {
                delete m_pRangeLimits;
            }
        };

    void SetReport (BOOL fReport) {m_fReport = fReport;};
    void SetRangeLimits (unsigned char NumRangeLimits, signed char *RangeLimits)
        {
            if ((NumRangeLimits > 0) && (RangeLimits != NULL)) {
                m_NumRangeLimits    = NumRangeLimits;
                m_pRangeLimits      = new signed char[NumRangeLimits];
                // copy RangeLimits
                for (unsigned char i = 0; i < NumRangeLimits; i++) {
                    m_pRangeLimits[i] = RangeLimits[i];
                }
            } else {
                m_NumRangeLimits = 0;
                m_pRangeLimits   = NULL;
            }
        };


    bool SendRequest (void) 
        {
            return CmdApi_SetEventReportReq (
                       (m_fReport ? true : false), 
                       m_NumRangeLimits, 
                       m_pRangeLimits
                   );
        }
private:
    BOOL            m_fReport;
    unsigned char   m_NumRangeLimits;
    signed char *   m_pRangeLimits;
};

class CReqGetPinStatus
{
public:
    CReqGetPinStatus () {};

    bool SendRequest (void) 
        {
            return CmdApi_GetPinStatusReq ();
        }
};

class CReqSetPinProtection
{
public:
    CReqSetPinProtection () {};
    ~CReqSetPinProtection () {};

    void SetPinId (unsigned int PinId) {m_PinId = PinId;};
    void SetProtectionSetting (BOOL ProtectionSetting) {m_ProtectionSetting = ProtectionSetting; };
    void SetPinValue (CString PinValue) {m_PinValue = PinValue; };

    bool SendRequest (void) 
        {
            return CmdApi_SetPinProtectionReq (
                       m_PinId,
                       (m_ProtectionSetting ? true : false),
                       m_PinValue.empty() ? NULL : (unsigned char *)m_PinValue.c_str()
                   );
        }

private:
    unsigned int    m_PinId;
    BOOL            m_ProtectionSetting;
    std::string     m_PinValue;
};

class CReqVerifyPin
{
public:
    CReqVerifyPin () {};
    ~CReqVerifyPin () {};

    void SetPinId (unsigned int PinId) {m_PinId = PinId;};
    void SetPinValue (CString PinValue) {m_PinValue = PinValue; };

    bool SendRequest (void) 
        {
            return CmdApi_VerifyPinReq (
                       m_PinId,
                       m_PinValue.empty() ? NULL : (unsigned char *)m_PinValue.c_str()
                   );
        }

private:
    unsigned int    m_PinId;
    std::string     m_PinValue;
};

class CReqServingSystem
{
public:
    CReqServingSystem () {};
    ~CReqServingSystem () {};

    bool SendRequest (void) 
        {
            return CmdApi_GetServingSystemReq ();
        }
};

class CCmdApi
{
public:
    CCmdApi(void);
    ~CCmdApi(void);
    static CCmdApi& GetInstance();

    virtual void NotifyMsg (CMessage* pMsg) {};

    BOOL EstablishDevice (CString DeviceName) 
        {
            return CmdApi_EstablishDevice((unsigned char *)DeviceName.GetBuffer());
        };

    BOOL TerminateDevice (void)
        {
            return CmdApi_TerminateDevice();
        };


    friend void CmdApiProcessMsg (PCMD_API_MSG_T  pCmdApiMsg);

private:
    void ProcessMsg (PCMD_API_MSG_T  pCmdApiMsg);

    CMessage *ProcessMsg_CMDAPI_MSGCLASS_NT_DEVICE  (PCMD_API_MSG_T  pCmdApiMsg);
    CMessage *ProcessMsg_CMDAPI_MSGCLASS_NT_MESSAGE (PCMD_API_MSG_T  pCmdApiMsg);

    static BOOL fCCmdApiExist;

};



