///////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2011 Cinterion Wireless Modules GmbH
//
// This software is protected by international intellectual property laws and
// treaties.
// Cinterion Wireless Modules GmbH ("Cinterion") grants to customer a limited,
// non-exclusive, non-transferable, royalty-free license to use the source code
// as a sample implementation and for the purpose of evaluation, analysis and
// to provide feedback thereon to Cinterion. Any right, title and interest in
// and to the source code, other than those expressly granted to the customer
// under this agreement, shall remain vested with Cinterion. This license may
// be terminated by Cinterion at any time in writing. Customer undertakes not
// to provide third parties with the source code. In particular, Customer is
// not allowed to sell the source code.
// The information contained in this document is considered the CONFIDENTIAL
// and PROPRIETARY information of Cinterion Wireless Modules GmbH and may not
// be disclosed or discussed with anyone who is not employed by Cinterion
// Wireless Modules GmbH, unless the individual/ company:
//
// i)  has an express need to know such information, and
//
// ii) disclosure of information is subject to the terms of a duly executed
//     Confidentiality and Non-Disclosure Agreement between Cinterion Wireless
//     Modules GmbH and the individual / company.
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
// EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
// CINTERION, ITS LEGAL REPRESENTATIVES AND VICARIOUS AGENTS SHALL 
// IRRESPECTIVE OF THE LEGAL GROUND  ONLY BE LIABLE FOR DAMAGES IF THE DAMAGE
// WAS CAUSED THROUGH CULPABLE BREACH OF A MAJOR CONTRACTUAL OBLIGATION
// (CARDINAL DUTY), I.E. A DUTY THE FULFILMENT OF WHICH ALLOWS THE PROPER
// EXECUTION OF THE RESPECTIVE AGREEMENT IN THE FIRST PLACE OR THE BREACH OF
// WHICH PUTS THE ACHIEVEMENT OF THE PURPOSE OF THE AGREEMENT AT STAKE,
// RESPECTIVELY, AND ON THE FULFILMENT OF WHICH THE RECIPIENT THEREFORE MAY
// RELY ON OR WAS CAUSED BY GROSS NEGLIGENCE OR INTENTIONALLY. ANY FURTHER
// LIABILITY FOR DAMAGES SHALL  IRRESPECTIVE OF THE LEGAL GROUND  BE
// EXCLUDED. IN THE EVENT THAT CINTERION IS LIABLE FOR THE VIOLATION OF A MAJOR
// CONTRACTUAL OBLIGATION IN THE ABSENCE OF GROSS NEGLIGENCE OR WILFUL CONDUCT,
// SUCH LIABILITY FOR DAMAGE SHALL BE LIMITED TO AN EXTENT WHICH, AT THE TIME
// WHEN THE RESPECTIVE AGREEMENT IS CONCLUDED, CINTERION SHOULD NORMALLY EXPECT
// TO ARISE DUE TO CIRCUMSTANCES THAT THE PARTIES HAD KNOWLEDGE OF AT SUCH
// POINT IN TIME. CINTERION SHALL IN NO EVENT BE LIABLE FOR INDIRECT AND
// CONSEQUENTIAL DAMAGES OR LOSS OF PROFIT. CINTERION SHALL IN NO EVENT BE
// LIABLE FOR AN AMOUNT EXCEEDING  20,000.00 PER EVENT OF DAMAGE. WITHIN THE
// BUSINESS RELATIONSHIP THE OVERALL LIABILITY SHALL BE LIMITED TO A TOTAL OF
//  100,000.00. CLAIMS FOR DAMAGES SHALL BECOME TIME-BARRED AFTER ONE YEAR AS
// OF THE BEGINNING OF THE STATUTORY LIMITATION PERIOD. IRRESPECTIVE OF THE
// LICENSEES KNOWLEDGE OR GROSS NEGLIGENT LACK OF KNOWLEDGE OF THE
// CIRCUMSTANCES GIVING RISE FOR A LIABILITY ANY CLAIMS SHALL BECOME
// TIME-BARRED AFTER FIVE YEARS AS OF THE LIABILITY AROSE. THE AFOREMENTIONED
// LIMITATION OR EXCLUSION OF LIABILITY SHALL NOT APPLY IN THE CASE OF CULPABLE
// INJURY TO LIFE, BODY OR HEALTH, IN CASE OF INTENTIONAL ACTS, UNDER THE
// LIABILITY PROVISIONS OF THE GERMAN PRODUCT LIABILITY ACT
// (PRODUKTHAFTUNGSGESETZ) OR IN CASE OF A CONTRACTUALLY AGREED OBLIGATION TO
// ASSUME LIABILITY IRRESPECTIVE OF ANY FAULT (GUARANTEE).
//
// IN THE EVENT OF A CONFLICT BETWEEN THE PROVISIONS OF THIS AGREEMENT AND
// ANOTHER AGREEMENT REGARDING THE SOURCE CODE (EXCEPT THE GENERAL TERMS AND
// CONDITIONS OF CINTERION) THE OTHER AGREEMENT SHALL PREVAIL.
//
// All rights created by patent grant or registration of a utility model or
// design patent are reserved.
//
///////////////////////////////////////////////////////////////////////////////
#include <sstream>
#include "stdafx.h"
#include "AboutDlg.h"
#include "VersionInfo.h"
#include "ConnectionManager.h"
#include "ConnectionManagerDlg.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#endif

// user defined windows message for new notice received
const static USHORT WM_NEW_NOTICE = WM_USER - 1;

// --------------------------------------------------------------------------
// Constructor
// --------------------------------------------------------------------------
ConnectionManagerDlg::ConnectionManagerDlg(UINT nIDCaption, CWnd* pParentWnd, UINT iSelectPage)
    :CPropertySheet(nIDCaption, pParentWnd, iSelectPage)
{
    AddConManPages();
}

ConnectionManagerDlg::ConnectionManagerDlg(LPCTSTR pszCaption, CWnd* pParentWnd, UINT iSelectPage)
	:CPropertySheet(pszCaption, pParentWnd, iSelectPage)
{
    AddConManPages();
}

// --------------------------------------------------------------------------
// Destructor
// --------------------------------------------------------------------------
ConnectionManagerDlg::~ConnectionManagerDlg()
{
    while (!m_MsgQueue.empty()) {
        CMessage* pMsg = m_MsgQueue.front();
        m_MsgQueue.pop();
        delete pMsg;
    }
}

void ConnectionManagerDlg::AddConManPages()
{
	m_hIcon = ::ExtractIcon(
                    AfxGetInstanceHandle(),
                    "conman.ico",
                    0
                );
    if (!m_hIcon)
	    m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);

	m_psh.dwFlags |= PSH_NOAPPLYNOW;     // Lose the Apply Now button
	m_psh.dwFlags &= ~PSH_HASHELP;  // Lose the Help button


	AddPage(&m_connectionPage);
	AddPage(&m_simPage);
}


BEGIN_MESSAGE_MAP(ConnectionManagerDlg, CPropertySheet)
	ON_WM_SYSCOMMAND()
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
    ON_WM_CLOSE()
    ON_MESSAGE(WM_POWERBROADCAST,OnPowerbroadcast)
    ON_MESSAGE(WM_NEW_NOTICE,OnNewNotice)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// ----------------------------------------------------------------------------
// OnInitDialog
// ----------------------------------------------------------------------------
BOOL ConnectionManagerDlg::OnInitDialog()
{
	// Add "About..." menu item to system menu.
	// IDM_ABOUTBOX must be in the system command range.
	ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
	ASSERT(IDM_ABOUTBOX < 0xF000);

	CMenu* pSysMenu = GetSystemMenu(FALSE);
    if (pSysMenu != NULL) {
		CString strAboutMenu;
		strAboutMenu.LoadString(IDS_ABOUTBOX);

        if (!strAboutMenu.IsEmpty()) {
			pSysMenu->AppendMenu(MF_SEPARATOR);
            strAboutMenu = "&About " + CString((VersionInfo::GetInstance().GetProductName()).c_str()) + "...";
			pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strAboutMenu);
		}
	}

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

    m_connectionPage.SetCMDlg(this);
    m_simPage.SetCMDlg(this);

    GetDlgItem(IDOK)->ShowWindow(false);
    GetDlgItem(IDCANCEL)->SetWindowText(_T("Close"));

	if (::AfxGetApp()->m_nCmdShow & SW_MINIMIZE) {
		// User wants app to start minimized, but that only affects the 
		// main window.  So, minimize this window as well.
		PostMessage(WM_SYSCOMMAND, SC_MINIMIZE);
	}
	
    CPropertySheet::OnInitDialog();

    SetActivePage(&m_simPage);
    SetActivePage(&m_connectionPage);

	return TRUE;
}

// ----------------------------------------------------------------------------
// OnClose
// ----------------------------------------------------------------------------
void ConnectionManagerDlg::OnClose()
{
    m_connectionPage.SaveUserData();
    CPropertySheet::OnClose();
}

// ----------------------------------------------------------------------------
// OnSysCommand
// ----------------------------------------------------------------------------
void ConnectionManagerDlg::OnSysCommand(UINT nID, LPARAM lParam)
{
	if ((nID & 0xFFF0) == IDM_ABOUTBOX) {
		AboutDlg dlgAbout;
		dlgAbout.DoModal();
	}
    CPropertySheet::OnSysCommand(nID, lParam);
}

// ----------------------------------------------------------------------------
// OnPaint
// ----------------------------------------------------------------------------
void ConnectionManagerDlg::OnPaint() 
{
	if (IsIconic())	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	} else {
		CPropertySheet::OnPaint();
	}
}

// ----------------------------------------------------------------------------
// OnQueryDragIcon
//
// The system calls this function to obtain the cursor to display while the 
// user drags the minimized window.
// ----------------------------------------------------------------------------
HCURSOR ConnectionManagerDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}

// --------------------------------------------------------------------------
// OnPowerbroadcast
//
// Actions to take when a powerbroadcast message is received from the PC.
// --------------------------------------------------------------------------
LRESULT ConnectionManagerDlg::OnPowerbroadcast(WPARAM wParam, LPARAM lParam)
{
    switch (wParam) {
        case PBT_APMSUSPEND:
            // terminate device on standby
            TerminateDevice();
            break;
    }

    m_connectionPage.OnPowerbroadcast(wParam, lParam);
    return NULL;
}

// --------------------------------------------------------------------------
// SelectDevice
//
// SelectDevice establishes the device name with the subsystem (ConManDll) and
// collects information about that device to be displayed.
//
// Parameters
// deviceName   : string name of the device to connect to.
// --------------------------------------------------------------------------
void ConnectionManagerDlg::SelectDevice(CString& deviceName)
{
    BeginWaitCursor();

    // call CCmdApi
    EstablishDevice (deviceName);
}

// --------------------------------------------------------------------------
// DeselectDevice
//
// DeselectDevice coordinates with subsystem to disable an active
// device and removes information about that device from the gui.
// --------------------------------------------------------------------------
void ConnectionManagerDlg::DeselectDevice()
{
    BeginWaitCursor();

    // call CCmdApi
    TerminateDevice();
}

// --------------------------------------------------------------------------
// NotifyMsg
//
// Implementation of the virtual function from the class CCmdApi.
// Actions to take when a CMessage is received from CCmdApi. Put
// the Message on the queue and post a message that a new notice
// has been received. 
// --------------------------------------------------------------------------
void ConnectionManagerDlg::NotifyMsg
(
    CMessage* pMsg
)
{
    // add the msg pointer to the msg queue
    m_MsgQueue.push(pMsg);

    while (m_hWnd == 0) Sleep(100);

    // post a message indicating that a new notice has been received
    ::PostMessage(m_hWnd, WM_NEW_NOTICE, NULL, NULL);
}

// --------------------------------------------------------------------------
// OnNewNotice
//
// Actions to take when a new notice has been placed on the message queue.
// --------------------------------------------------------------------------
LRESULT ConnectionManagerDlg::OnNewNotice(WPARAM wParam,LPARAM lParam)
{
    CMessage*   pMsg;
    pMsg = m_MsgQueue.front();
    m_MsgQueue.pop();

    // handle CMessage;
    switch (pMsg->GetClass()) {
        case CMDAPI_MSGCLASS_NT_DEVICE:
            ProcessDeviceMsg ((CMsgDevice *)pMsg);
            break;

        case CMDAPI_MSGCLASS_NT_MESSAGE:
            ProcessMessage ((CMessage *)pMsg);
            break;

    }

    // destroy CMessage object
    delete pMsg;

    return NULL;
}

// --------------------------------------------------------------------------
// ProcessDeviceMsg
//
// Actions to take when a DeviceNotice is received from the CCmdApi.
//
// Parameters
// pMsg     : pointer of CMessage object to process.
// --------------------------------------------------------------------------
void ConnectionManagerDlg::ProcessDeviceMsg(CMsgDevice *pMsg)
{
    switch (pMsg->GetType())
    {
        case CMDAPI_MSG_DT_AVAILABLE:
            // update the available devices
            m_connectionPage.PopulateDeviceNames(pMsg->GetNetworkAdapters());
            break;

        case CMDAPI_MSG_DT_CONNECT:
           
            // direct pages to enable appropriate fields
            m_connectionPage.EnableSelectedState();
            m_simPage.EnableSelectedState();

            EndWaitCursor();

            // simulate a connect if AutoConnect is set
            m_connectionPage.AutoConnect();

            break;

        case CMDAPI_MSG_DT_CONNECT_FAIL:

            // direct pages to disable appropriate fields
            m_connectionPage.DisableSelectedState();
            m_simPage.DisableSelectedState();

            EndWaitCursor();

            // do all work before displaying message box
            AfxMessageBox(pMsg->GetReason().c_str());
            break;

        case CMDAPI_MSG_DT_DISCONNECT:

            // direct pages to disable appropriate fields
            m_connectionPage.DisableSelectedState();
            m_simPage.DisableSelectedState();

            EndWaitCursor();
            break;

        case CMDAPI_MSG_DT_DETACH:

            // update the available devices
            m_connectionPage.PopulateDeviceNames(pMsg->GetNetworkAdapters());
            
            // direct pages to disable appropriate fields
            m_connectionPage.DisableSelectedState();
            m_simPage.DisableSelectedState();

            // do all work before displaying message box
            AfxMessageBox(pMsg->GetReason().c_str());
            break;

        default:
            AfxMessageBox(_T("Error: Unknown device message received."));
    }
}

// --------------------------------------------------------------------------
// ProcessMessage
//
// Actions to take when a CMessage (Rsp/Ind) is received from the CCmdApi.
//
// Parameters
// pMsg     : pointer of CMessage object to process.
// --------------------------------------------------------------------------
void ConnectionManagerDlg::ProcessMessage(CMessage *pMsg)
{
    switch (pMsg->GetType()) {
        case CMDAPI_MSG_MT_START_CONNECTION_RSP:
            ProcessMsgStartConnectionRsp ((CMsgStartConnectionRsp *)pMsg);
            break;
        case CMDAPI_MSG_MT_ABORT_CONNECTION_RSP:
            ProcessMsgAbortConnectionRsp ((CMsgAbortConnectionRsp *)pMsg);
            break;
        case CMDAPI_MSG_MT_STOP_CONNECTION_RSP:
            ProcessMsgStopConnectionRsp ((CMsgStopConnectionRsp *)pMsg);
            break;
        case CMDAPI_MSG_MT_STATUS_CONNECTION_IND:
            ProcessMsgStatusConnectionInd ((CMsgStatusConnectionInd *)pMsg);
            break;
        case CMDAPI_MSG_MT_SIGNAL_STRENGTH_RSP:
            ProcessMsgGetSignalStrengthRsp ((CGetSignalStrengthRsp *)pMsg);
            break;
        case CMDAPI_MSG_MT_SET_EVENT_REPORT_RSP:
            ProcessMsgSetEventReportRsp ((CSetEventReportRsp *)pMsg);
            break;
        case CMDAPI_MSG_MT_EVENT_REPORT_IND:
            ProcessMsgEventReportInd ((CEventReportInd *)pMsg);
            break;
        case CMDAPI_MSG_MT_GET_PIN_STATUS_RSP:
            ProcessMsgGetPinStatusRsp((CGetPinStatusRsp *)pMsg);
            break;
        case CMDAPI_MSG_MT_SET_PIN_PROTECTION_RSP:
            ProcessMsgSetPinProtectionRsp((CSetPinProtectionRsp *)pMsg);
            break;
        case CMDAPI_MSG_MT_VERIFY_PIN_RSP:
            ProcessMsgVerifyPinRsp((CVerifyPinRsp *)pMsg);
            break;
        case CMDAPI_MSG_MT_SERVING_SYSTEM_RSP:
            ProcessMsgServingSystemRsp((CServingSystemRsp *)pMsg);
            break;
        case CMDAPI_MSG_MT_SERVING_SYSTEM_IND:
            ProcessMsgServingSystemInd((CServingSystemInd *)pMsg);
            break;

        default:
            break;
    }
    return;
}

// --------------------------------------------------------------------------
// ProcessMsgStartConnectionRsp
//
// Actions to take when a CMDAPI_MSG_MT_START_CONNECTION_RSP is received.
//
// Parameters
// pMsg     : pointer of CMsgStartConnectionRsp object to process.
// --------------------------------------------------------------------------
void ConnectionManagerDlg::ProcessMsgStartConnectionRsp(CMsgStartConnectionRsp *pMsg)
{
    // verify message was successful
    if (pMsg->GetResult() != CMDAPI_RESULT_SUCCESS)
    {
        ReportMsgFailed(_T("StartConnectionReq"), pMsg->GetError());

        // call Page disable methods
        m_connectionPage.DisableConnectedState();
        return;
    }

    // call Page enable methods
    m_connectionPage.EnableConnectedState(pMsg->GetDataHandle());
}

// --------------------------------------------------------------------------
// ProcessMsgAbortConnectionRsp
//
// Actions to take when a CMDAPI_MSG_MT_ABORT_CONNECTION_RSP is received.
//
// Parameters
// pMsg     : pointer of CMsgAbortConnectionRsp object to process.
// --------------------------------------------------------------------------
void ConnectionManagerDlg::ProcessMsgAbortConnectionRsp(CMsgAbortConnectionRsp *pMsg)
{
    // verify message was successful
    if (pMsg->GetResult() != CMDAPI_RESULT_SUCCESS)
    {
        ReportMsgFailed(_T("AbortConnectionReq"), pMsg->GetError());
        return;
    }

    // call Page enable methods
    m_connectionPage.DisableConnectedState();
}

// --------------------------------------------------------------------------
// ProcessMsgStopConnectionRsp
//
// Actions to take when a CMDAPI_MSG_MT_STOP_CONNECTION_RSP is received.
//
// Parameters
// pMsg     : pointer of CMsgStopConnectionRsp pointer to process.
// --------------------------------------------------------------------------
void ConnectionManagerDlg::ProcessMsgStopConnectionRsp(CMsgStopConnectionRsp *pMsg)
{
    // verify message was successful
    if (pMsg->GetResult() != CMDAPI_RESULT_SUCCESS)
    {
        ReportMsgFailed(_T("StopConnectionReq"), pMsg->GetError());
        return;
    }

    // call Page disable methods
    m_connectionPage.DisableConnectedState();
}

// --------------------------------------------------------------------------
// ProcessMsgStatusConnectionInd
//
// Actions to take when a CMDAPI_MSG_MT_STATUS_CONNECTION_IND is received.
//
// Parameters
// pMsg     : pointer of CMsgStatusConnectionInd pointer to process.
// --------------------------------------------------------------------------
void ConnectionManagerDlg::ProcessMsgStatusConnectionInd(CMsgStatusConnectionInd *pMsg)
{
    // call ConnectionPage process method
    m_connectionPage.ProcessMsgStatusConnectionInd(pMsg);
}

// --------------------------------------------------------------------------
// ProcessMsgGetSignalStrengthRsp
//
// Actions to take when a CMDAPI_MSG_MT_SIGNAL_STRENGTH_RSP message is received.
//
// Parameters
// pMsg     : pointer of CGetSignalStrengthRsp pointer to process.
// --------------------------------------------------------------------------
void ConnectionManagerDlg::ProcessMsgGetSignalStrengthRsp(CGetSignalStrengthRsp *pMsg)
{
    // verify message was successful
    if (pMsg->GetResult() != CMDAPI_RESULT_SUCCESS)
    {
        ReportMsgFailed(_T("GetSignalStrengthReq"), pMsg->GetError());
        return;
    }

    // call Page process methods
    m_connectionPage.ProcessMsgGetSignalStrengthRsp(pMsg);
}

// --------------------------------------------------------------------------
// ProcessMsgSetEventReportRsp
//
// Actions to take when a CMDAPI_MSG_MT_SET_EVENT_REPORT_RSP is received.
//
// Parameters
// pMsg     : pointer of CSetEventReportRsp pointer to process.
// --------------------------------------------------------------------------
void ConnectionManagerDlg::ProcessMsgSetEventReportRsp(CSetEventReportRsp *pMsg)
{
    // verify message was successful
    if (pMsg->GetResult() != CMDAPI_RESULT_SUCCESS)
    {
        ReportMsgFailed(_T("SetEventReportReq"), pMsg->GetError());
        return;
    }
}

// --------------------------------------------------------------------------
// ProcessMsgGetPinStatusRsp
//
// Actions to take when a CMDAPI_MSG_MT_GET_PIN_STATUS_RSP is received.
//
// Parameters
// pMsg     : pointer of CGetPinStatusRsp pointer to process.
// --------------------------------------------------------------------------
void ConnectionManagerDlg::ProcessMsgGetPinStatusRsp(CGetPinStatusRsp *pMsg)
{
    // verify message was successful
    if (pMsg->GetResult() != CMDAPI_RESULT_SUCCESS)
    {
        ReportMsgFailed(_T("GetPinStatusReq"), pMsg->GetError());
        return;
    }

    // call Page process methods
    m_simPage.ProcessGetPinStatusRsp (pMsg);
}


// --------------------------------------------------------------------------
// ProcessMsgSetPinProtectionRsp
//
// Actions to take when a CMDAPI_MSG_MT_SET_PIN_PROTECTION_RSP is received.
//
// Parameters
// pMsg     : pointer of CSetPinProtectionRsp pointer to process.
// --------------------------------------------------------------------------
void ConnectionManagerDlg::ProcessMsgSetPinProtectionRsp(CSetPinProtectionRsp *pMsg)
{
    // verify message was successful
    if (pMsg->GetResult() != CMDAPI_RESULT_SUCCESS)
    {
        ReportMsgFailed(_T("SetPinProtectionReq"),pMsg->GetError());
        return;
    }
}

// --------------------------------------------------------------------------
// ProcessMsgVerifyPinRsp
//
// Actions to take when a CMDAPI_MSG_MT_VERIFY_PIN_RSP is received.
//
// Parameters
// pMsg     : pointer of CVerifyPinRsp pointer to process.
// --------------------------------------------------------------------------
void ConnectionManagerDlg::ProcessMsgVerifyPinRsp(CVerifyPinRsp *pMsg)
{
    // verify message was successful
    if (pMsg->GetResult() != CMDAPI_RESULT_SUCCESS)
    {
        ReportMsgFailed(_T("VerifyPinReq"),pMsg->GetError());
        return;
    }
}

// --------------------------------------------------------------------------
// ProcessMsgEventReportInd
//
// Actions to take when a CMDAPI_MSG_MT_EVENT_REPORT_IND message is received.
//
// Parameters
// pMsg     : pointer of CEventReportInd pointer to process.
// --------------------------------------------------------------------------
void ConnectionManagerDlg::ProcessMsgEventReportInd(CEventReportInd *pMsg)
{
    // call ConnectionPage process method
    m_connectionPage.ProcessMsgEventReportInd(pMsg);
}

// --------------------------------------------------------------------------
// ProcessMsgServingSystemRsp
//
// Actions to take when a CMDAPI_MSG_MT_SERVING_SYSTEM_RSP is received.
//
// Parameters
// pMsg     : pointer of CServingSystemRsp pointer to process.
// --------------------------------------------------------------------------
void ConnectionManagerDlg::ProcessMsgServingSystemRsp(CServingSystemRsp *pMsg)
{
    // verify message was successful
    if (pMsg->GetResult() != CMDAPI_RESULT_SUCCESS)
    {
        ReportMsgFailed(_T("ServingSystemReq"),pMsg->GetError());
        return;
    }

    // call ConnectionPage process method
    m_connectionPage.ProcessMsgServingSystemRsp(pMsg);
}

// --------------------------------------------------------------------------
// ProcessMsgServingSystemInd
//
// Actions to take when a CMDAPI_MSG_MT_SERVING_SYSTEM_IND is received.
//
// Parameters
// pMsg     : pointer of CServingSystemInd pointer to process.
// --------------------------------------------------------------------------
void ConnectionManagerDlg::ProcessMsgServingSystemInd(CServingSystemInd *pMsg)
{
    // call ConnectionPage process method
    m_connectionPage.ProcessMsgServingSystemInd(pMsg);
}


// --------------------------------------------------------------------------
// ReportMsgFailed
//
// Report a message failure. Map the eroor code to text.
//
// Parameters
// name     : user defined text
// dwError  : error code
// --------------------------------------------------------------------------
void ConnectionManagerDlg::ReportMsgFailed(std::string name, DWORD dwError)
{
    std::stringstream stream;

    if (dwError < CMDAPI_ERR_MAX)
    {
        stream << _T("Error: ") << name << _T(" failed with error '")
               << CMDAPI_ERROR_STRINGS[dwError] << _T("'.");
    }
    else
    {
        stream << _T("Error: ") << name << _T(" failed with error '")
               << dwError << _T("'.");
    }
    
    AfxMessageBox(stream.str().c_str());
}

