///////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2011 Cinterion Wireless Modules GmbH
//
// This software is protected by international intellectual property laws and
// treaties.
// Cinterion Wireless Modules GmbH ("Cinterion") grants to customer a limited,
// non-exclusive, non-transferable, royalty-free license to use the source code
// as a sample implementation and for the purpose of evaluation, analysis and
// to provide feedback thereon to Cinterion. Any right, title and interest in
// and to the source code, other than those expressly granted to the customer
// under this agreement, shall remain vested with Cinterion. This license may
// be terminated by Cinterion at any time in writing. Customer undertakes not
// to provide third parties with the source code. In particular, Customer is
// not allowed to sell the source code.
// The information contained in this document is considered the CONFIDENTIAL
// and PROPRIETARY information of Cinterion Wireless Modules GmbH and may not
// be disclosed or discussed with anyone who is not employed by Cinterion
// Wireless Modules GmbH, unless the individual/ company:
//
// i)  has an express need to know such information, and
//
// ii) disclosure of information is subject to the terms of a duly executed
//     Confidentiality and Non-Disclosure Agreement between Cinterion Wireless
//     Modules GmbH and the individual / company.
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
// EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
// CINTERION, ITS LEGAL REPRESENTATIVES AND VICARIOUS AGENTS SHALL 
// IRRESPECTIVE OF THE LEGAL GROUND  ONLY BE LIABLE FOR DAMAGES IF THE DAMAGE
// WAS CAUSED THROUGH CULPABLE BREACH OF A MAJOR CONTRACTUAL OBLIGATION
// (CARDINAL DUTY), I.E. A DUTY THE FULFILMENT OF WHICH ALLOWS THE PROPER
// EXECUTION OF THE RESPECTIVE AGREEMENT IN THE FIRST PLACE OR THE BREACH OF
// WHICH PUTS THE ACHIEVEMENT OF THE PURPOSE OF THE AGREEMENT AT STAKE,
// RESPECTIVELY, AND ON THE FULFILMENT OF WHICH THE RECIPIENT THEREFORE MAY
// RELY ON OR WAS CAUSED BY GROSS NEGLIGENCE OR INTENTIONALLY. ANY FURTHER
// LIABILITY FOR DAMAGES SHALL  IRRESPECTIVE OF THE LEGAL GROUND  BE
// EXCLUDED. IN THE EVENT THAT CINTERION IS LIABLE FOR THE VIOLATION OF A MAJOR
// CONTRACTUAL OBLIGATION IN THE ABSENCE OF GROSS NEGLIGENCE OR WILFUL CONDUCT,
// SUCH LIABILITY FOR DAMAGE SHALL BE LIMITED TO AN EXTENT WHICH, AT THE TIME
// WHEN THE RESPECTIVE AGREEMENT IS CONCLUDED, CINTERION SHOULD NORMALLY EXPECT
// TO ARISE DUE TO CIRCUMSTANCES THAT THE PARTIES HAD KNOWLEDGE OF AT SUCH
// POINT IN TIME. CINTERION SHALL IN NO EVENT BE LIABLE FOR INDIRECT AND
// CONSEQUENTIAL DAMAGES OR LOSS OF PROFIT. CINTERION SHALL IN NO EVENT BE
// LIABLE FOR AN AMOUNT EXCEEDING  20,000.00 PER EVENT OF DAMAGE. WITHIN THE
// BUSINESS RELATIONSHIP THE OVERALL LIABILITY SHALL BE LIMITED TO A TOTAL OF
//  100,000.00. CLAIMS FOR DAMAGES SHALL BECOME TIME-BARRED AFTER ONE YEAR AS
// OF THE BEGINNING OF THE STATUTORY LIMITATION PERIOD. IRRESPECTIVE OF THE
// LICENSEES KNOWLEDGE OR GROSS NEGLIGENT LACK OF KNOWLEDGE OF THE
// CIRCUMSTANCES GIVING RISE FOR A LIABILITY ANY CLAIMS SHALL BECOME
// TIME-BARRED AFTER FIVE YEARS AS OF THE LIABILITY AROSE. THE AFOREMENTIONED
// LIMITATION OR EXCLUSION OF LIABILITY SHALL NOT APPLY IN THE CASE OF CULPABLE
// INJURY TO LIFE, BODY OR HEALTH, IN CASE OF INTENTIONAL ACTS, UNDER THE
// LIABILITY PROVISIONS OF THE GERMAN PRODUCT LIABILITY ACT
// (PRODUKTHAFTUNGSGESETZ) OR IN CASE OF A CONTRACTUALLY AGREED OBLIGATION TO
// ASSUME LIABILITY IRRESPECTIVE OF ANY FAULT (GUARANTEE).
//
// IN THE EVENT OF A CONFLICT BETWEEN THE PROVISIONS OF THIS AGREEMENT AND
// ANOTHER AGREEMENT REGARDING THE SOURCE CODE (EXCEPT THE GENERAL TERMS AND
// CONDITIONS OF CINTERION) THE OTHER AGREEMENT SHALL PREVAIL.
//
// All rights created by patent grant or registration of a utility model or
// design patent are reserved.
//
///////////////////////////////////////////////////////////////////////////////
#pragma once

#include <string>
#include <vector>
#include "CmdApi.h"

class ConnectionManagerDlg;


// --------------------------------------------------------------------------
// ConnectionPage
//
/// The ConnectionPage class represents the controls under the connection
/// tab of the ConnectionManager dialog. Input from the user and output to
/// the user under the connection tab is controlled by this class. 
// --------------------------------------------------------------------------

class ConnectionPage : public CPropertyPage
{
public:
    ConnectionPage();
    void SetCMDlg(ConnectionManagerDlg* pCMDlg);
    void SetTxId(UINT txId) { m_txId = txId; }
    void EnableSelectedState();
    void DisableSelectedState();
    void EnableConnectedState(UINT pktDataHandle);
    void DisableConnectedState();
    void PopulateDeviceNames(std::vector<std::string>& networkAdapters);

    void ProcessMsgGetSignalStrengthRsp(CGetSignalStrengthRsp *pMsg);
    void ProcessMsgEventReportInd(CEventReportInd *pMsg);

    void ProcessAttachState(BYTE CSAttachState, BYTE PSAttachState);
    void ProcessMsgServingSystemRsp(CServingSystemRsp *pMsg);
    void ProcessMsgServingSystemInd(CServingSystemInd *pMsg);
    void ProcessMsgStatusConnectionInd (CMsgStatusConnectionInd * pMsg);

    LRESULT OnPowerbroadcast(WPARAM wParam,LPARAM lParam);

    void AutoConnect(void);
    void SaveUserData(void);

protected:
	virtual void DoDataExchange(CDataExchange* pDX);
	virtual BOOL OnInitDialog();
    afx_msg BOOL OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message);
    afx_msg void OnDeviceSelect();
    afx_msg void OnPrimaryDnsCheck();
    afx_msg void OnSecondaryDnsCheck();
    afx_msg void OnPrimaryNbnsCheck();
    afx_msg void OnSecondaryNbnsCheck();
    afx_msg void OnApnNameCheck();
    afx_msg void OnIpAddressCheck();
    afx_msg void OnAuthPrefCheck();
    afx_msg void OnUsernameCheck();
    afx_msg void OnPasswordCheck();
    afx_msg void OnConnect();

    void EnableAbortState();
    void ProcessSignalStrength(INT signalStrength, BYTE radioIf);
    
    void ReqStartConnection(CReqStartConnection& msg);
    void ReqSetEventReport(CReqSetEventReport& msg);


	DECLARE_MESSAGE_MAP()

private:
    ConnectionManagerDlg* m_pCMDlg;
    bool m_useWaitCursor;

    CStatic m_selectDeviceStatic;
    CComboBox m_deviceNameCombo;
    CStatic m_overrideGoup;
    CButton m_primaryDnsCheck;
    CIPAddressCtrl m_primaryDnsIpAddr;
    CButton m_secondaryDnsCheck;
    CIPAddressCtrl m_secondaryDnsIpAddr;
    CButton m_primaryNbnsCheck;
    CIPAddressCtrl m_primaryNbnsIpAddr;
    CButton m_secondaryNbnsCheck;
    CIPAddressCtrl m_secondaryNbnsIpAddr;
    CButton m_apnNameCheck;
    CEdit m_apnNameEdit;
    CButton m_ipAddressCheck;
    CIPAddressCtrl m_ipAddressIpAddr;
    CButton m_authPrefCheck;
    CComboBox m_authPrefCombo;
    CButton m_usernameCheck;
    CEdit m_usernameEdit;
    CButton m_passwordCheck;
    CEdit m_passwordEdit;
    CButton m_connectButton;
    CStatic m_signalStatic;
    CStatic m_RFTech;
    CProgressCtrl m_signalProgress;
    CButton m_autoconnectCheck;

    CStatic m_CSAttachStateStatic;
    CStatic m_PSAttachStateStatic;
    CStatic m_RegStateStatic;


    CString m_deviceName;
    bool m_isSelected;
    bool m_isConnected;
    bool m_isPBT_APMSUSPEND;
    UINT m_pktDataHandle;
    UINT m_txId;
};
