///////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2011 Cinterion Wireless Modules GmbH
//
// This software is protected by international intellectual property laws and
// treaties.
// Cinterion Wireless Modules GmbH ("Cinterion") grants to customer a limited,
// non-exclusive, non-transferable, royalty-free license to use the source code
// as a sample implementation and for the purpose of evaluation, analysis and
// to provide feedback thereon to Cinterion. Any right, title and interest in
// and to the source code, other than those expressly granted to the customer
// under this agreement, shall remain vested with Cinterion. This license may
// be terminated by Cinterion at any time in writing. Customer undertakes not
// to provide third parties with the source code. In particular, Customer is
// not allowed to sell the source code.
// The information contained in this document is considered the CONFIDENTIAL
// and PROPRIETARY information of Cinterion Wireless Modules GmbH and may not
// be disclosed or discussed with anyone who is not employed by Cinterion
// Wireless Modules GmbH, unless the individual/ company:
//
// i)  has an express need to know such information, and
//
// ii) disclosure of information is subject to the terms of a duly executed
//     Confidentiality and Non-Disclosure Agreement between Cinterion Wireless
//     Modules GmbH and the individual / company.
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
// EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
// CINTERION, ITS LEGAL REPRESENTATIVES AND VICARIOUS AGENTS SHALL 
// IRRESPECTIVE OF THE LEGAL GROUND  ONLY BE LIABLE FOR DAMAGES IF THE DAMAGE
// WAS CAUSED THROUGH CULPABLE BREACH OF A MAJOR CONTRACTUAL OBLIGATION
// (CARDINAL DUTY), I.E. A DUTY THE FULFILMENT OF WHICH ALLOWS THE PROPER
// EXECUTION OF THE RESPECTIVE AGREEMENT IN THE FIRST PLACE OR THE BREACH OF
// WHICH PUTS THE ACHIEVEMENT OF THE PURPOSE OF THE AGREEMENT AT STAKE,
// RESPECTIVELY, AND ON THE FULFILMENT OF WHICH THE RECIPIENT THEREFORE MAY
// RELY ON OR WAS CAUSED BY GROSS NEGLIGENCE OR INTENTIONALLY. ANY FURTHER
// LIABILITY FOR DAMAGES SHALL  IRRESPECTIVE OF THE LEGAL GROUND  BE
// EXCLUDED. IN THE EVENT THAT CINTERION IS LIABLE FOR THE VIOLATION OF A MAJOR
// CONTRACTUAL OBLIGATION IN THE ABSENCE OF GROSS NEGLIGENCE OR WILFUL CONDUCT,
// SUCH LIABILITY FOR DAMAGE SHALL BE LIMITED TO AN EXTENT WHICH, AT THE TIME
// WHEN THE RESPECTIVE AGREEMENT IS CONCLUDED, CINTERION SHOULD NORMALLY EXPECT
// TO ARISE DUE TO CIRCUMSTANCES THAT THE PARTIES HAD KNOWLEDGE OF AT SUCH
// POINT IN TIME. CINTERION SHALL IN NO EVENT BE LIABLE FOR INDIRECT AND
// CONSEQUENTIAL DAMAGES OR LOSS OF PROFIT. CINTERION SHALL IN NO EVENT BE
// LIABLE FOR AN AMOUNT EXCEEDING  20,000.00 PER EVENT OF DAMAGE. WITHIN THE
// BUSINESS RELATIONSHIP THE OVERALL LIABILITY SHALL BE LIMITED TO A TOTAL OF
//  100,000.00. CLAIMS FOR DAMAGES SHALL BECOME TIME-BARRED AFTER ONE YEAR AS
// OF THE BEGINNING OF THE STATUTORY LIMITATION PERIOD. IRRESPECTIVE OF THE
// LICENSEES KNOWLEDGE OR GROSS NEGLIGENT LACK OF KNOWLEDGE OF THE
// CIRCUMSTANCES GIVING RISE FOR A LIABILITY ANY CLAIMS SHALL BECOME
// TIME-BARRED AFTER FIVE YEARS AS OF THE LIABILITY AROSE. THE AFOREMENTIONED
// LIMITATION OR EXCLUSION OF LIABILITY SHALL NOT APPLY IN THE CASE OF CULPABLE
// INJURY TO LIFE, BODY OR HEALTH, IN CASE OF INTENTIONAL ACTS, UNDER THE
// LIABILITY PROVISIONS OF THE GERMAN PRODUCT LIABILITY ACT
// (PRODUKTHAFTUNGSGESETZ) OR IN CASE OF A CONTRACTUALLY AGREED OBLIGATION TO
// ASSUME LIABILITY IRRESPECTIVE OF ANY FAULT (GUARANTEE).
//
// IN THE EVENT OF A CONFLICT BETWEEN THE PROVISIONS OF THIS AGREEMENT AND
// ANOTHER AGREEMENT REGARDING THE SOURCE CODE (EXCEPT THE GENERAL TERMS AND
// CONDITIONS OF CINTERION) THE OTHER AGREEMENT SHALL PREVAIL.
//
// All rights created by patent grant or registration of a utility model or
// design patent are reserved.
//
///////////////////////////////////////////////////////////////////////////////
#pragma once

#include <windows.h>
#include <queue>
#include <deque>

//////////////////////////////////////////////////////////////////////////////

// Class for creating blocks of code that are wrapped in a critical section
class SYNCBLOCK_Implementation {
public:
    SYNCBLOCK_Implementation(CRITICAL_SECTION& rcs) { m_pcs = &rcs; EnterCriticalSection(m_pcs); }
    ~SYNCBLOCK_Implementation() { LeaveCriticalSection(m_pcs); }
private:
    CRITICAL_SECTION* m_pcs;
};

// Class for creating blocks of code inside a SYNCBLOCK that are NOT wrapped in a critical section
class UNSYNCBLOCK_Implementation {
public:
    UNSYNCBLOCK_Implementation(CRITICAL_SECTION& rcs) { m_pcs = &rcs; LeaveCriticalSection(m_pcs); }
    ~UNSYNCBLOCK_Implementation() { EnterCriticalSection(m_pcs); }
private:
    CRITICAL_SECTION* m_pcs;
};

// Macros to make SYNCBLOCK and UNSYNCBLOCK easier to use
#define SYNCBLOCK(cs)                SYNCBLOCK_Implementation    SYNCBLOCK_Object(cs)
#define UNSYNCBLOCK(cs)              UNSYNCBLOCK_Implementation  UNSYNCBLOCK_Object(cs)


// ----------------------------------------------------------------------------
// MsgQueue
//
// A MsgQueue is a thread-safe wrapper around a std::queue. 
// ----------------------------------------------------------------------------
template<class _Typ, class _Container = std::deque<_Typ> >
class MsgQueue
{
public:
    typedef _Container container_type;
	typedef typename _Container::value_type value_type;
	typedef typename _Container::size_type size_type;

    MsgQueue() 
    {
        InitializeCriticalSection(&m_csLock);
    }

    ~MsgQueue()
    {
        DeleteCriticalSection(&m_csLock);
    }


    bool empty() 
    {
        SYNCBLOCK(m_csLock);
        return m_queue.empty();
    }

    size_type size() 
    {
        SYNCBLOCK(m_csLock);
        return m_queue.size();
    }

    value_type& front()
    {
        SYNCBLOCK(m_csLock);
        return m_queue.front();
    }

    value_type& back() 
    {
        SYNCBLOCK(m_csLock);
        return m_queue.back();
    }

    void push(const value_type& _Val)
    {
        SYNCBLOCK(m_csLock);
        m_queue.push(_Val);
    }

    void pop()
    {
        SYNCBLOCK(m_csLock);
        m_queue.pop();
    }


private:
    CRITICAL_SECTION    m_csLock;
    std::queue<_Typ>     m_queue;

};



