///////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2011 Cinterion Wireless Modules GmbH
//
// This software is protected by international intellectual property laws and
// treaties.
// Cinterion Wireless Modules GmbH ("Cinterion") grants to customer a limited,
// non-exclusive, non-transferable, royalty-free license to use the source code
// as a sample implementation and for the purpose of evaluation, analysis and
// to provide feedback thereon to Cinterion. Any right, title and interest in
// and to the source code, other than those expressly granted to the customer
// under this agreement, shall remain vested with Cinterion. This license may
// be terminated by Cinterion at any time in writing. Customer undertakes not
// to provide third parties with the source code. In particular, Customer is
// not allowed to sell the source code.
// The information contained in this document is considered the CONFIDENTIAL
// and PROPRIETARY information of Cinterion Wireless Modules GmbH and may not
// be disclosed or discussed with anyone who is not employed by Cinterion
// Wireless Modules GmbH, unless the individual/ company:
//
// i)  has an express need to know such information, and
//
// ii) disclosure of information is subject to the terms of a duly executed
//     Confidentiality and Non-Disclosure Agreement between Cinterion Wireless
//     Modules GmbH and the individual / company.
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
// EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
// CINTERION, ITS LEGAL REPRESENTATIVES AND VICARIOUS AGENTS SHALL 
// IRRESPECTIVE OF THE LEGAL GROUND  ONLY BE LIABLE FOR DAMAGES IF THE DAMAGE
// WAS CAUSED THROUGH CULPABLE BREACH OF A MAJOR CONTRACTUAL OBLIGATION
// (CARDINAL DUTY), I.E. A DUTY THE FULFILMENT OF WHICH ALLOWS THE PROPER
// EXECUTION OF THE RESPECTIVE AGREEMENT IN THE FIRST PLACE OR THE BREACH OF
// WHICH PUTS THE ACHIEVEMENT OF THE PURPOSE OF THE AGREEMENT AT STAKE,
// RESPECTIVELY, AND ON THE FULFILMENT OF WHICH THE RECIPIENT THEREFORE MAY
// RELY ON OR WAS CAUSED BY GROSS NEGLIGENCE OR INTENTIONALLY. ANY FURTHER
// LIABILITY FOR DAMAGES SHALL  IRRESPECTIVE OF THE LEGAL GROUND  BE
// EXCLUDED. IN THE EVENT THAT CINTERION IS LIABLE FOR THE VIOLATION OF A MAJOR
// CONTRACTUAL OBLIGATION IN THE ABSENCE OF GROSS NEGLIGENCE OR WILFUL CONDUCT,
// SUCH LIABILITY FOR DAMAGE SHALL BE LIMITED TO AN EXTENT WHICH, AT THE TIME
// WHEN THE RESPECTIVE AGREEMENT IS CONCLUDED, CINTERION SHOULD NORMALLY EXPECT
// TO ARISE DUE TO CIRCUMSTANCES THAT THE PARTIES HAD KNOWLEDGE OF AT SUCH
// POINT IN TIME. CINTERION SHALL IN NO EVENT BE LIABLE FOR INDIRECT AND
// CONSEQUENTIAL DAMAGES OR LOSS OF PROFIT. CINTERION SHALL IN NO EVENT BE
// LIABLE FOR AN AMOUNT EXCEEDING  20,000.00 PER EVENT OF DAMAGE. WITHIN THE
// BUSINESS RELATIONSHIP THE OVERALL LIABILITY SHALL BE LIMITED TO A TOTAL OF
//  100,000.00. CLAIMS FOR DAMAGES SHALL BECOME TIME-BARRED AFTER ONE YEAR AS
// OF THE BEGINNING OF THE STATUTORY LIMITATION PERIOD. IRRESPECTIVE OF THE
// LICENSEES KNOWLEDGE OR GROSS NEGLIGENT LACK OF KNOWLEDGE OF THE
// CIRCUMSTANCES GIVING RISE FOR A LIABILITY ANY CLAIMS SHALL BECOME
// TIME-BARRED AFTER FIVE YEARS AS OF THE LIABILITY AROSE. THE AFOREMENTIONED
// LIMITATION OR EXCLUSION OF LIABILITY SHALL NOT APPLY IN THE CASE OF CULPABLE
// INJURY TO LIFE, BODY OR HEALTH, IN CASE OF INTENTIONAL ACTS, UNDER THE
// LIABILITY PROVISIONS OF THE GERMAN PRODUCT LIABILITY ACT
// (PRODUKTHAFTUNGSGESETZ) OR IN CASE OF A CONTRACTUALLY AGREED OBLIGATION TO
// ASSUME LIABILITY IRRESPECTIVE OF ANY FAULT (GUARANTEE).
//
// IN THE EVENT OF A CONFLICT BETWEEN THE PROVISIONS OF THIS AGREEMENT AND
// ANOTHER AGREEMENT REGARDING THE SOURCE CODE (EXCEPT THE GENERAL TERMS AND
// CONDITIONS OF CINTERION) THE OTHER AGREEMENT SHALL PREVAIL.
//
// All rights created by patent grant or registration of a utility model or
// design patent are reserved.
//
///////////////////////////////////////////////////////////////////////////////

#include <windows.h>
#include "VersionInfo.h"
#include <vector>
#include <sstream>
#include <iomanip>

#pragma comment ( lib, "Version.lib" )

VersionInfo& VersionInfo::GetInstance()
{
   static VersionInfo versionInfo;
   return versionInfo;
}

VersionInfo::VersionInfo(const std::string& filename)
: m_filename(filename)
{
    if(m_filename.empty()) {
        TCHAR thisPath[MAX_PATH];
        if(0 < ::GetModuleFileName(NULL, thisPath, MAX_PATH)) {
            m_filename = thisPath;

            // build conman.ini filename
            m_inifilename = m_filename;
            m_inifilename = m_inifilename.replace(m_inifilename.length()-3, m_inifilename.length(), "ini");
        }
    }
}

VersionInfo::~VersionInfo()
{
}

//-----------------------------------------------------------------------------
// GetVersionInfo
//
// Copy the Version Info resource from the executable to the given buffer
//-----------------------------------------------------------------------------
bool GetVersionInfo(const std::string& filename, std::vector<BYTE>& buffer)
{
    if(filename.empty()) {
        return false;
    }
   
    // Get the size (in bytes) of the version resource for the given file
    DWORD dwHandle = 0;
    DWORD dwInfoSize = GetFileVersionInfoSize(const_cast<LPTSTR>(filename.c_str()), &dwHandle);

    // If the size is zero, we couldn't get the resource
    if(0 == dwInfoSize) {
        return false;
    }

    // Copy the version resource into the buffer
    buffer.resize(dwInfoSize, 0);
    if(!GetFileVersionInfo(const_cast<LPTSTR>(filename.c_str()), dwHandle, dwInfoSize, &buffer[0])) {
        return false;
    }
    return true;
}

//-----------------------------------------------------------------------------
// GetStringTableData
//
// This method gets a string from a string table in the VS_VERSION_INFO
// resource for the module specified by m_filename
//-----------------------------------------------------------------------------
std::string VersionInfo::GetStringTableData(const std::string& csEntry)
{
    std::vector<BYTE> versionInfo;
    if(!::GetVersionInfo(m_filename, versionInfo)) {
        return "Failed to get version info";
    }

    TCHAR loc[] = TEXT("\\StringFileInfo\\040904B0\\");
    std::vector<TCHAR> entry(loc, loc + sizeof(loc)/sizeof(loc[0]));
    entry.resize(entry.size() + csEntry.size() + 1, '\0');
    strncat(&entry[0], csEntry.c_str(), entry.size());
      
    UINT strLen = 0;
    LPCTSTR pValue = NULL;
    if(!VerQueryValue(&versionInfo[0], &entry[0], (void**)&pValue, &strLen)) {
        return "Could not find " + csEntry;
    }
    return pValue;
}

//-----------------------------------------------------------------------------
// GetFixedVersionInfo
//
// This function gets Version Information from 
// FIXED FILE INFO block in the VS_VERSION_INFO resource
// for the specified module
//-----------------------------------------------------------------------------
std::string VersionInfo::GetFixedVersionInfo(const std::string& csEntry)
{
    std::vector<BYTE> versionInfo;
    if(!::GetVersionInfo(m_filename, versionInfo)) {
        return "GetVersionInfo Failed";
    }

    TCHAR label[] = TEXT("\\");
    UINT fixedLength = 0;
    VS_FIXEDFILEINFO* pFixedInfo = NULL;
    if(!VerQueryValue(&versionInfo[0], label, (void**)&pFixedInfo, &fixedLength)) {
        return "Failed to find fixed version info";
    }

    std::ostringstream os;
    if(csEntry == "ProductVersion") {
        os << std::setw(1) << std::setfill(TEXT('0')) << HIWORD(pFixedInfo->dwProductVersionMS) << '.'
           << std::setw(1) << std::setfill(TEXT('0')) << LOWORD(pFixedInfo->dwProductVersionMS) << '.'
           << std::setw(1) << std::setfill(TEXT('0')) << HIWORD(pFixedInfo->dwProductVersionLS) << '.'
           << std::setw(1) << std::setfill(TEXT('0')) << LOWORD(pFixedInfo->dwProductVersionLS);
    } else {
        os << std::setw(1) << std::setfill(TEXT('0')) << HIWORD(pFixedInfo->dwFileVersionMS) << '.'
           << std::setw(1) << std::setfill(TEXT('0')) << LOWORD(pFixedInfo->dwFileVersionMS) << '.'
           << std::setw(1) << std::setfill(TEXT('0')) << HIWORD(pFixedInfo->dwFileVersionLS) << '.'
           << std::setw(1) << std::setfill(TEXT('0')) << LOWORD(pFixedInfo->dwFileVersionLS);
    }
    return os.str();
}


std::string  VersionInfo::GetProductVersion()
{
    return GetFixedVersionInfo("ProductVersion");
}

std::string VersionInfo::GetFileVersion()
{
    return GetFixedVersionInfo("FileVersion");
}

std::string  VersionInfo::GetProductName()
{
    DWORD dwCnt = 0;
    TCHAR ProductName[1024];

    dwCnt = ::GetPrivateProfileString(
                "ConMan",
                "ProductName",
                "",
                (LPTSTR)ProductName,
                sizeof(ProductName),
                (LPCTSTR)m_inifilename.c_str()
              );
    if (dwCnt) {
        return ProductName;
    } else {
        return GetStringTableData("ProductName");
    }
}

std::string  VersionInfo::GetCompanyName()
{
    DWORD dwCnt = 0;
    TCHAR CompanyName[1024];

    dwCnt = ::GetPrivateProfileString(
                "ConMan",
                "CompanyName",
                "Manufacturer",
                (LPTSTR)CompanyName,
                sizeof(CompanyName),
                (LPCTSTR)m_inifilename.c_str()
              );

    if (dwCnt) {
        return CompanyName;
    } else {
        return GetStringTableData( "CompanyName");
    }
}

std::string VersionInfo::GetLegalCopyright()
{
    return GetStringTableData("LegalCopyright");
}

std::string  VersionInfo::GetFileDescription()
{
    DWORD dwCnt = 0;
    TCHAR ProductName[1024];

    dwCnt = ::GetPrivateProfileString(
                "ConMan",
                "ProductName",
                "",
                (LPTSTR)ProductName,
                sizeof(ProductName),
                (LPCTSTR)m_inifilename.c_str()
              );
    if (dwCnt) {
        return ProductName;
    } else {
        return GetStringTableData("FileDescription");
    }
}
